@@ -10,6 +10,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include "defs.h"
+#include "extern.h"
 #include "const.h"
 #include "function.h"
 
@@ -48,6 +49,9 @@ int main(int argc, char **argv) {
     }
     /* ե륯 */
     fclose(ofp);
+    free(bpx);
+    free(noise_comp);
+    free(imgd1.image);
     if(!nlines)
         free(mlineseg);
 #ifdef TIME
