@@ -19,28 +19,42 @@
 // File: bpnet.cc
 // Purpose: neural network classifier
 // Responsible: Hagen Kaprykowsky (kapry@iupr.net)
-// Reviewer:
+// Reviewer: Yves Rangoni (rangoni@iupr.net)
 // Primary Repository:
 // Web Sites: www.iupr.org, www.dfki.de
 
+#include <float.h>
 #include "bpnet.h"
 #include "ocr-utils.h"
 #include "confusion-matrix.h"
 
+#include "narray-ops.h"
+#include "logger.h"
+
 using namespace ocropus;
 using namespace colib;
+using namespace narray_ops;
 
 namespace iupr_bpnet {
 
 #define MIN_SCORE 1e-6
 #define epsilon_stdev 1e-04
 
+    // log the confusion matrix for the training set for each epoch
+    Logger logger_confusion_train("confusion_train");
+    // log the confusion matrix for the testing set for each epoch
+    Logger logger_confusion_test("confusion_test");
+
+    // log error rates for each epoch
+    Logger logger_errors("error_rates");
+
+    param_int freq_error_rates("freq_error_rates",1,"frequency of error_rates graphs plotting");
+
     template<class T>
     bool valid(T &v) {
         for(int i=0;i<v.length1d();i++)
             if(isnan(v.at1d(i))) {
-            	//printf("[");print(v);printf("]\n");
-                return false;                
+                return false;
             }
         return true;
     }
@@ -79,8 +93,10 @@ namespace iupr_bpnet {
     }
 
     // not to be used outside this scope
-#define NSIGMOID 10000
-#define SIGMOID_RANGE 15.0
+    //#define NSIGMOID 10000
+    //#define SIGMOID_RANGE 15.0
+    static const int NSIGMOID=10000;
+    static const float SIGMOID_RANGE=15.0;
 
     // table of the sigmoid function
     static floatarray sigmoid_table(NSIGMOID);
@@ -92,8 +108,7 @@ namespace iupr_bpnet {
 
     // initialize the simgoid table
     static void init_sigmoid_table() {
-        int i;
-        for(i=0;i<NSIGMOID;i++) {
+        for(int i=0;i<NSIGMOID;i++) {
             sigmoid_table(i) = slow_sigmoid(i*SIGMOID_RANGE/NSIGMOID);
         }
     }
@@ -109,22 +124,13 @@ namespace iupr_bpnet {
             inited = true;
         }
 
-        if(abs_x>=20.0) {
+        if(abs_x>=SIGMOID_RANGE) {
             abs_result = 1.0;
         } else {
             index = int(NSIGMOID*abs_x/SIGMOID_RANGE);
-            if(index>=NSIGMOID) {
-                abs_result = 1.0;
-            } else {
-                abs_result = sigmoid_table(index);
-            }
-        }
-
-        if(x<0.0) {
-            return 1.0-abs_result;
-        } else {
-            return abs_result;
+            abs_result = (index>=NSIGMOID) ? 1.0 : sigmoid_table(index);
         }
+        return (x<0.0) ? 1.0-abs_result:abs_result;
     }
 
     // float random number between low and high
@@ -140,7 +146,6 @@ namespace iupr_bpnet {
                                   floatarray &weights,floatarray &offsets,
                                   bool &sigmoid_inited) {
         float total;
-
         for(int i=0;i<noutput;i++) {
             total = offsets(i);
             for(int j=0;j<ninput;j++) {
@@ -155,7 +160,6 @@ namespace iupr_bpnet {
                              floatarray &activations_input,
                              floatarray &delta_output,int ninput,
                              floatarray &weights) {
-
         for(int j=0;j<ninput;j++) {
             float deriv = activations_input(j)*(1.0-activations_input(j));
             float total = 0.0;
@@ -167,11 +171,9 @@ namespace iupr_bpnet {
     }
 
     // weight update using backpropagation formula
-    void bp_update_weights(floatarray &offsets,floatarray &delta_output,int
-                           noutput,floatarray &activation_input,int ninput,
-                           floatarray &weights,float eta) {
-
-
+    void bp_update_weights(floatarray &offsets,floatarray &delta_output,
+                            int noutput,floatarray &activation_input,int ninput,
+                            floatarray &weights,float eta) {
         for(int i=0;i<noutput;i++) {
             for(int j=0;j<ninput;j++) {
                 weights(i,j) += eta*delta_output(i)*activation_input(j);
@@ -184,62 +186,55 @@ namespace iupr_bpnet {
     // down to mean=0, std_dev=1
     void normalize_input_train(objlist<floatarray> &vectors,doublearray
                                &stdev,doublearray &m_x) {
-	
         CHECK_ARG(stdev.length()==m_x.length());
         int nsamples = vectors.length();
-		CHECK_CONDITION(nsamples>0);
+        CHECK_CONDITION(nsamples>0);
         int ninput = m_x.length();
         doublearray m_xx;
         m_xx.resize(ninput);
         fill(m_xx,0.0f);
         fill(m_x,0.0f);
         fill(stdev,0.0f);
-        
-        for(int d=0;d<ninput;d++) {
 
+        for(int d=0;d<ninput;d++) {
             // calc mean and empirical variance
             for(int n=0;n<nsamples;n++) {
                 m_x(d)  += vectors[n](d);
             }
             m_x(d) /= nsamples;
             for(int n=0;n<nsamples;n++) {
-            	double t = vectors[n](d) - m_x(d);
+                double t = vectors[n](d) - m_x(d);
                 m_xx(d) += t * t;
-            }            
+            }
             m_xx(d) /= nsamples;
-			double sqr_stdev = m_xx(d);
-			if (sqr_stdev < 0.)
-				sqr_stdev = 0.;
+            double sqr_stdev = m_xx(d);
+            if(sqr_stdev < 0.)
+                sqr_stdev = 0.;
             stdev(d) = sqrt(sqr_stdev);
             // normalize
             for(int n=0;n<nsamples;n++) {
                 if(stdev(d)>epsilon_stdev) {
                     vectors[n](d) = (vectors[n](d)-m_x(d))/stdev(d);
-                } else {
-                    vectors[n](d) = vectors[n](d)-m_x(d);        // var = 0: all the same;
+                } else {    // var = 0: all the same;
+                    vectors[n](d) = vectors[n](d)-m_x(d);
                 }
             }
 
         } // end dim loop
         ASSERT(valid(m_x));
-       	ASSERT(valid(stdev));
+        ASSERT(valid(stdev));
     } // end normalize_input
 
     void normalize_input_retrain(objlist<floatarray> &vectors,doublearray
                                  &stdev,doublearray &m_x) {
-
         CHECK_ARG(stdev.length()==m_x.length());
         int ninput = m_x.length();
         int nsamples = vectors.length();
-
         for(int d=0;d<ninput;d++) {
-
-            // normalize
             for(int n=0;n<nsamples;n++) {
                 if(stdev(d)>epsilon_stdev) {
                     vectors[n](d) = (vectors[n](d)-m_x(d))/stdev(d);
-                }
-                else {
+                } else {
                     vectors[n](d) = vectors[n](d)-m_x(d); //var=0: all the same;
                 }
             }
@@ -250,7 +245,7 @@ namespace iupr_bpnet {
 #undef NSIGMOID
 #undef SIGMOID_RANGE
 
-    class BpnetClassifier : public Classifier {
+    class BpnetClassifier : public IClassifier {
     public:
         objlist<floatarray> vectors;
         narray<int> classes;
@@ -277,19 +272,23 @@ namespace iupr_bpnet {
         int epochs;
         float learningrate;
         float testportion;
-        autodel<ConfusionMatrix> confusion;
-
+        autodel<ConfusionMatrix> confusion_train;
+        autodel<ConfusionMatrix> confusion_test;
+        autodel<ConfusionMatrix> best_confusion_train;
+        autodel<ConfusionMatrix> best_confusion_test;
         bool filedump;
         stdio fp;
-        char buf[1000];
+        char buf[4096];
         narray<float> weights_hidden_input_best;
         narray<float> hidden_offsets_best;
         narray<float> weights_output_hidden_best;
         narray<float> output_offsets_best;
 
-        // from Classifier
-        BpnetClassifier() {
-            filedump = false;
+        float eta_plus,eta_minus,eta_tol;
+        bool vlr;
+
+        void common_presets() {
+
             init = false;
             sigmoid_inited = false;
             training = false;
@@ -301,22 +300,22 @@ namespace iupr_bpnet {
             epochs = -1;
             learningrate = -1.0f;
             testportion = 0.0f;
+
+            eta_plus = 1.10;
+            eta_minus = 0.6;
+            eta_tol = 1.02;
+            vlr = false;
         }
 
-        BpnetClassifier(const char* path) {
-            strncpy(buf,path,sizeof(buf));
+        BpnetClassifier() {
+            common_presets();
+            filedump = false;
+        }
+
+        BpnetClassifier(const char* path_tmp_mlp) {
+            common_presets();
+            strncpy(buf,path_tmp_mlp,sizeof(buf));
             filedump = true;
-            init = false;
-            sigmoid_inited = false;
-            training = false;
-            norm = true;
-            shuffle = true;
-            ninput = -1;
-            nhidden = -1;
-            noutput = -1;
-            epochs = -1;
-            learningrate = -1.0f;
-            testportion = 0.0f;
         }
 
         void param(const char *name,double value) {
@@ -329,6 +328,11 @@ namespace iupr_bpnet {
             else if(!strcmp(name,"normalize")) norm = bool(value);
             else if(!strcmp(name,"shuffle")) shuffle = bool(value);
             else if(!strcmp(name,"filedump")) filedump = bool(value);
+
+            else if(!strcmp(name,"vlr")) vlr = bool(value);
+            else if(!strcmp(name,"eta_plus")) eta_plus = float(value);
+            else if(!strcmp(name,"eta_minus")) eta_minus= float(value);
+            else if(!strcmp(name,"eta_tol")) eta_tol = float(value);
             else throw "unknown parameter name";
         }
 
@@ -352,25 +356,35 @@ namespace iupr_bpnet {
                     noutput = max(classes) + 1;
                 create();
                 if(norm) {
+                    printf("Normalizing... ");fflush(stdout);logger_errors("Normalizing");
+
                     if(init) {
                         normalize_input_retrain(vectors,stdev,m_x);
-                    }
-                    else {
+                    } else {
                         normalize_input_train(vectors,stdev,m_x);
                     }
                 }
+                printf("end\n");
                 if(shuffle) {
+                    printf("Shuffling... ");fflush(stdout);logger_errors("Shuffling");
                     shuffle_feat(vectors,classes);
+                    printf("end\n");
                 }
                 init_backprop(0.001);
                 train();
                 dealloc_train();
                 init = true;
             }
-
             training = false;
         }
 
+        void start_classifying( intarray &confusion_matrix_train_out,
+                                intarray &confusion_matrix_test_out) {
+            start_classifying();
+            best_confusion_train->get(confusion_matrix_train_out);
+            best_confusion_test->get(confusion_matrix_test_out);
+        }
+
         void seal() {
             vectors.dealloc();
             classes.dealloc();
@@ -390,12 +404,7 @@ namespace iupr_bpnet {
             forward();
             // Copy output layer to result avoiding scores less than MIN_SCORE
             for(int i=0;i<output.length();i++) {
-                if(output(i)<MIN_SCORE) {
-                    result(i) = MIN_SCORE;
-                }
-                else {
-                    result(i) = output(i);
-                }
+                result(i) = max(MIN_SCORE, output(i));
             }
         }
 
@@ -407,7 +416,7 @@ namespace iupr_bpnet {
             fprintf(stream,"bp3-net %d %d %d %d\n",ninput,nhidden,noutput,norm);
             if(norm) {
                 // write mean and variance
-                for (int d=0;d<ninput;d++) {
+                for(int d=0;d<ninput;d++) {
                     fprintf(stream,"%f %f\n",m_x(d),stdev(d));
                 }
                 ASSERT(valid(m_x));
@@ -426,16 +435,14 @@ namespace iupr_bpnet {
             if(!stream) {
                 throw "bad input format";
             }
-
             if(fscanf(stream,"bp3-net %d %d %d %d",&ninput,&nhidden,&noutput,&norm_tmp)!=4 ||
-               ninput<1||ninput>1000000||nhidden<1||nhidden>1000000||noutput<1||
-               noutput>1000000) {
+                ninput<1||ninput>1000000||nhidden<1||nhidden>1000000||noutput<1||
+                noutput>1000000) {
                 throw "bad input format";
             }
             norm = bool(norm_tmp);
             create();
-            if(norm) {
-                // read mean and variance
+            if(norm) {  // read mean and variance
                 for(int d=0;d<ninput;d++) {
                     fscanf(stream,"%lf %lf\n",&m_x_tmp,&stdev_tmp);
                     m_x(d) = m_x_tmp;
@@ -460,95 +467,239 @@ namespace iupr_bpnet {
                 input.resize(ninput);
                 hidden.resize(nhidden);
                 output.resize(noutput);
-                confusion = make_ConfusionMatrix(noutput);
+                confusion_train = make_ConfusionMatrix(noutput);
+                confusion_test = make_ConfusionMatrix(noutput);
+                best_confusion_train = make_ConfusionMatrix(noutput);
+                best_confusion_test = make_ConfusionMatrix(noutput);
                 m_x.resize(ninput);
                 stdev.resize(ninput);
             }
         }
 
+        void forward_one_input(int sample_index, int &target, int &predicted) {
+
+            copy(input,vectors[sample_index]);
+            target = classes(sample_index);
+
+            forward();
+            predicted = argmax(output);
+            ASSERT(predicted>=0 && predicted<noutput);
+        }
+
+        void test_on_db(int n1, int n2, float &_error, float &cls_error,
+            ConfusionMatrix& cm) {
+            ASSERT(n2>n1);
+            int cls, predicted;
+            _error = 0.0;
+            cls_error = 0.0;
+            cm.clear();
+            for(int sample_index=n1;sample_index<n2;sample_index++) {
+                forward_one_input(sample_index, cls, predicted);
+                compute_error(cls);
+                _error += norm2squared(error);
+                cls_error += (predicted!=cls);
+                cm.increment(cls,predicted);
+            }
+            cls_error /= (float(n2-n1));
+        }
+
+        void logging_train_test(const char* str,float _error,float cls_error,
+            ConfusionMatrix& cm,int ep,float best_cls_error,float best_error) {
+            char temp_string[4096];
+            if(strcmp(str,"Train")==0) {
+                sprintf(temp_string, "Train cls error: %10.4f%%\t"
+                        "Train error: %10.4f\n",cls_error*100.,_error);
+                logger_confusion_train.format(  "Train confusion matrix"
+                                                " for epoch %d",ep);
+                logger_confusion_train.confusion(cm);
+            } else {
+                sprintf(temp_string, "Test cls error:     %7.4f%%\t"
+                                        "Test error:  %10.4f\t"
+                                        "Best test cls error: %7.4f%%\t"
+                                        "Best test error: %10.4f\n",
+                                        cls_error*100.,_error,
+                                        best_cls_error*100., best_error);
+                logger_confusion_test.format(   "Test confusion matrix"
+                                                " for epoch %d",ep);
+                logger_confusion_test.confusion(cm);
+            }
+            printf(temp_string);logger_errors(temp_string);
+        }
+
+
         void train() {
             CHECK_CONDITION(learningrate>0.0f);
             ASSERT(testportion>=0.0f&&testportion<=1.0f);
+
+            floatarray histo_train_error;histo_train_error.resize(epochs+1);
+            floatarray histo_train_cls_error;histo_train_cls_error.resize(epochs+1);
+            floatarray histo_test_error;histo_test_error.resize(epochs+1);
+            floatarray histo_test_cls_error;histo_test_cls_error.resize(epochs+1);
+            floatarray histo_lr;histo_lr.resize(epochs+1);
+
+            char temp_string[1024];
             int predicted = -1;
             int cls = -1;
-            float trainerror;
-            float testerror;
-            float besttesterror = 1000000.0f;
-            float besttrainerror = 1000000.0f;
-            int ntrain = int(float(vectors.length())*(1.0f-testportion));
-            int ntest = vectors.length()-ntrain;
-            float E;
-			printf("ep:%d nh:%d lr:%g tp:%g\n", epochs, nhidden, learningrate, testportion);
-            printf("=== Start training on %d samples for %d epochs ===\n",ntrain,epochs);
+            float train_cls_error;
+            //float best_train_cls_error = FLT_MAX;
+            float train_error;
+            float best_train_error = FLT_MAX;
+
+            float test_cls_error = 0.0f;
+            float best_test_cls_error = FLT_MAX;
+            float test_error = 0.0f;
+            float best_test_error = FLT_MAX;
+
+            int n = vectors.length();
+            int ntrain = int(float(n)*(1.0f-testportion));
+            int ntest = n-ntrain;
+
+            sprintf(temp_string, "ep:%d nh:%d lr:%g tp:%g np:%d i:%d o:%d\n",
+                    epochs,nhidden,learningrate,testportion,n,ninput,noutput);
+            printf(temp_string);logger_errors(temp_string);
+
+            sprintf(temp_string, "=== Start training on %d samples "
+                    "(testing on %d) for %d epochs ===\n",ntrain,ntest,epochs);
+            printf(temp_string);logger_errors(temp_string);
+
+            //local copy of the current mlp
+            floatarray weights_hidden_input_temp;
+            floatarray hidden_offsets_temp;
+            floatarray weights_output_hidden_temp;
+            floatarray output_offsets_temp;
+
+            float old_train_error = 0.0f;
+            sprintf(temp_string, "Epoch: %d\tLR: %f\n\n",0,learningrate);
+            printf(temp_string);logger_errors(temp_string);
+
+            test_on_db(0,ntrain,train_error,train_cls_error,*confusion_train);
+            old_train_error = best_train_error = train_error;
+            logging_train_test( "Train",train_error,train_cls_error,
+                                *confusion_train,-1,0,0);
+
+            test_on_db(ntrain,n,test_error,test_cls_error,*confusion_test);
+            best_test_error = test_error;
+            best_test_cls_error = test_cls_error;
+            logging_train_test( "Test ",test_error,test_cls_error,
+                                *confusion_test,-1,
+                                best_test_cls_error,best_test_error);
+
+            histo_train_error[0] = train_error/ntrain;
+            histo_train_cls_error[0] = train_cls_error*100.;
+            histo_test_error[0] = test_error/ntest;
+            histo_test_cls_error[0] = test_cls_error*100.;
+            histo_lr[0] = learningrate;
+
             for(int epoch=0;epoch<epochs;epoch++) {
-                printf("Epoch: %d\n",epoch);
-                confusion->clear();
-                trainerror = 0.0f;
-                E = 0.0f;
+                sprintf(temp_string, "Epoch: %d\tLR: %f\n",epoch+1,learningrate);
+                printf(temp_string);logger_errors(temp_string);
+                //confusion_test->printReduced(stdout);
+                //keep current network in memory
+                copy(weights_hidden_input_temp, weights_hidden_input);
+                copy(hidden_offsets_temp,       hidden_offsets);
+                copy(weights_output_hidden_temp,weights_output_hidden);
+                copy(output_offsets_temp,       output_offsets);
+
+                //train
                 for(int sample_index=0;sample_index<ntrain;sample_index++) {
-                    copy(input,vectors[sample_index]);
-                    forward();
-                    for(int k=0;k<noutput;k++) {
-                        E += error(k)*error(k);
-                    }
-                    backward(classes(sample_index));
+                    forward_one_input(sample_index, cls, predicted);
+                    compute_error(cls);
+                    backward();
                     update();
-                    predicted = argmax(output);
-                    ASSERT(predicted>=0&&predicted<noutput);
-                    cls = classes(sample_index);
-                    confusion->increment(cls,predicted);
-                    trainerror += (predicted!=cls);
+                    if(sample_index % 1000 == 0) {
+                        printf("%d/%d\r", sample_index, ntrain);fflush(stdout);
+                    }
                 }
-                trainerror /= (float(ntrain));
-                printf("Training error: %f \t E: %f\n",trainerror,E);
+                printf("\n");
+
+                test_on_db(0,ntrain,train_error,train_cls_error,*confusion_train);
+                logging_train_test( "Train",train_error,train_cls_error,
+                                    *confusion_train,epoch,0,0);
+
+                //test
                 if(testportion>0.0f) {
-                    confusion->clear();
-                    testerror = 0.0f;
-                    for(int sample_index=ntrain;sample_index<vectors.length();sample_index++) {
-                        copy(input,vectors[sample_index]);
-                        forward();
-                        predicted = argmax(output);
-                        ASSERT(predicted>=0&&predicted<noutput);
-                        cls = classes(sample_index);
-                        confusion->increment(cls,predicted);
-                        testerror += (predicted!=cls);
+                    test_on_db(ntrain,n,test_error,test_cls_error,*confusion_test);
+
+                    if(test_cls_error < best_test_cls_error) {
+                        best_test_cls_error = test_cls_error;
                     }
-                    testerror /= (float(ntest));
-                    if(besttesterror>testerror) {
+                    if(test_error < best_test_error) {
                         if(filedump) {
                             copy(weights_hidden_input_best,weights_hidden_input);
                             copy(hidden_offsets_best,hidden_offsets);
                             copy(weights_output_hidden_best,weights_output_hidden);
-                            copy(output_offsets_best,hidden_offsets);
-                            save(stdio(buf, "w"));
+                            copy(output_offsets_best,output_offsets);
+                            save(stdio(buf,"w"));
                         }
-                        besttesterror = testerror;
+                        best_test_error = test_error;
+                        best_confusion_train->set(*confusion_train);
+                        best_confusion_test->set(*confusion_test);
                     }
-                    printf("Test error:     %f \t Best test error: %f\n",testerror,besttesterror);
-                }
-                else {
-                    if(besttrainerror>trainerror) {
+                    logging_train_test( "Test",test_error,test_cls_error,
+                                        *confusion_test,epoch,
+                                        best_test_cls_error,best_test_error);
+                    histo_test_error[epoch+1] = test_error/ntest;
+                    histo_test_cls_error[epoch+1] = test_cls_error*100;
+                } else {
+                    if(train_error < best_train_error) {
                         if(filedump) {
                             copy(weights_hidden_input_best,weights_hidden_input);
                             copy(hidden_offsets_best,hidden_offsets);
                             copy(weights_output_hidden_best,weights_output_hidden);
-                            copy(output_offsets_best,hidden_offsets);
-                            save(stdio(buf, "w"));
+                            copy(output_offsets_best,output_offsets);
+                            save(stdio(buf,"w"));
                         }
-                        besttrainerror = trainerror;
+                        best_train_error = train_error;
+                        best_confusion_train->set(*confusion_train);
                     }
                 }
+                histo_train_error[epoch+1] = train_error/ntrain;
+                histo_train_cls_error[epoch+1] = train_cls_error*100.;
+                histo_lr[epoch+1] = learningrate;
+
+                if(vlr) {
+                    if(train_error / old_train_error >= eta_tol) {
+                        learningrate *= eta_minus;
+                        printf("eta -\n");
+                        logger_errors("eta -");
+                        copy(weights_hidden_input,  weights_hidden_input_temp);
+                        copy(hidden_offsets,        hidden_offsets_temp);
+                        copy(weights_output_hidden, weights_output_hidden_temp);
+                        copy(output_offsets,        output_offsets_temp);
+                    } else {
+                        if(train_error < old_train_error) {
+                            learningrate *= eta_plus;
+                            printf("eta +\n");
+                            logger_errors("eta +");
+                        } else {
+                            printf("eta =\n");
+                            logger_errors("eta =");
+                        }
+                        old_train_error = train_error;
+                    }
+                }
+                if((freq_error_rates>0) && ((epoch%freq_error_rates)==0 || (epoch==0)))
+                logger_errors.train_test_curves(
+                    histo_train_error,histo_train_cls_error,
+                    histo_test_error,histo_test_cls_error,
+                    histo_lr,epoch+1);
+                //confusion_test->printReduced(stdout);
             }
-            if(filedump) {
+            if(filedump) {  // get the best weights
                 copy(weights_hidden_input,weights_hidden_input_best);
                 copy(hidden_offsets,hidden_offsets_best);
                 copy(weights_output_hidden,weights_output_hidden_best);
-                copy(output_offsets,hidden_offsets_best);
+                copy(output_offsets,output_offsets_best);
             }
+            fflush(stdout);
+
+            logger_errors.train_test_curves(
+                    histo_train_error,histo_train_cls_error,
+                    histo_test_error,histo_test_cls_error,
+                    histo_lr,epochs);
         }
 
         void init_backprop(float range) {
-
             error.resize(noutput);
             hidden_deltas.resize(nhidden);
             output_deltas.resize(noutput);
@@ -592,10 +743,13 @@ namespace iupr_bpnet {
                                      sigmoid_inited);
         }
 
-        void backward(int cls) {
-            int i;
+        void compute_error(int cls) {
             copy(error,output);
             error(cls) -= 1;
+        }
+
+        void backward() {
+            int i;
             for(i=0;i<noutput;i++) {
                 float deriv = output(i)*(1.0-output(i));
                 output_deltas(i) = -deriv*error(i);
@@ -613,16 +767,15 @@ namespace iupr_bpnet {
                               input,ninput,weights_hidden_input,
                               learningrate);
         }
-
     };
 }
 
 namespace ocropus {
-    Classifier *make_BpnetClassifier() {
+    IClassifier *make_BpnetClassifier() {
         using namespace iupr_bpnet;
         return new BpnetClassifier();
     }
-    Classifier *make_BpnetClassifierDumpIntoFile(const char *path) {
+    IClassifier *make_BpnetClassifierDumpIntoFile(const char *path) {
         using namespace iupr_bpnet;
         return new BpnetClassifier(path);
     }
