@@ -19,7 +19,7 @@
 // File: bpnet.h
 // Purpose: Neural network classifier
 // Responsible: Hagen Kaprykowsky (kapry@iupr.net)
-// Reviewer:
+// Reviewer: Yves Rangoni (rangoni@iupr.org)
 // Primary Repository:
 // Web Sites: www.iupr.org, www.dfki.de
 
@@ -33,8 +33,12 @@
 
 namespace ocropus {
     /// Create a bpnet (standard backpropagation MLP) classifier.
-    colib::Classifier *make_BpnetClassifier();
+    colib::IClassifier *make_BpnetClassifier();
 
-    colib::Classifier *make_BpnetClassifierDumpIntoFile(const char *path);
+    /// Create a bpnet (standard backpropagation MLP) classifier
+    /// After each epoch the current state of the MLP is saved
+    /// into the file path_mlp_dump_into_file
+    colib::IClassifier *make_BpnetClassifierDumpIntoFile(
+                                    const char *path_mlp_dump_into_file);
 };
 #endif
