@@ -32,13 +32,21 @@
 namespace ocropus {
 
     colib::ICharacterClassifier *make_AdaptClassifier(
-        colib::Classifier *,
-        bool output_garbage = false);
+        colib::IClassifier *,
+        bool output_garbage = false,
+        const char* features = "111101110",
+        int dim_x = 10,
+        int dim_y = 10);
+
+    colib::ICharacterClassifier *make_BpnetCharacterClassifier(
+        const char* features = "111101010",
+        int dim_x = 10,
+        int dim_y = 10);
 
     void train(colib::ICharacterClassifier &, ICharacterLibrary &);
 /*    void train(colib::ICharacterClassifier &,
                const char *path_file_list_segmentation,
-               const char *path_file_list_grid, 
+               const char *path_file_list_grid,
                bool garbage = false);*/
 
 }
