@@ -23,14 +23,19 @@
 
 #include "feature-extractor.h"
 #include "imglib.h"
+#include "checks.h"
 #include "ocr-utils.h"
+//#include "peel.cc"
+#include "logger.h"
 
 using namespace colib;
-using namespace imglib;
+using namespace iulib;
 using namespace ocropus;
 
-#define DIM_X 10
-#define DIM_Y 10
+namespace {
+    Logger logger("fe");
+}
+
 #define STANDARD_Y 40
 #define MAX_REL_SIZE 1.5
 #define IN(i,j,image) ((i>=0)&&(i<(image).dim(0))&&(j>=0)&&(j<(image).dim(1)))
@@ -186,7 +191,7 @@ float interpolate_bilinear(bytearray &img, float x, float y) {
 
 // scale box from source image to box in dest image
 void scale_box_gray_from_gray(bytearray &dst, rectangle bDst,
-                              bytearray &src,rectangle bSrc) {
+                              bytearray &src, rectangle bSrc) {
     int w = bDst.width();
     int h = bDst.height();
     float u,v;
@@ -209,7 +214,7 @@ void scale_box_gray_from_gray(bytearray &dst, rectangle bDst,
 }
 
 // scale box from source image to box in dest image
-void scale_box_gray(bytearray &dst, rectangle bDst, 
+void scale_box_gray(bytearray &dst, rectangle bDst,
                     bytearray &src, rectangle bSrc) {
     int w = bDst.width();
     int h = bDst.height();
@@ -642,15 +647,42 @@ void preprocess(bytearray &image) {
     }
 }
 
+void remove_isolated_pixels(colib::bytearray &out,colib::bytearray &in,float d,byte T) {
+    int window = min(in.dim(0),in.dim(1));
+    window = max(window,1);
+    window = int(window * d +.5);
+    window = max(window,1);
+
+    copy(out,in);
+    for (int i=0;i<in.dim(0);i++) {
+        for (int j=0;j<in.dim(1);j++) {
+            int isolated = 0;
+            for (int u=-window;u<=window;u++) {
+                for (int w=-window;w<=window;w++) {
+                    if (in(min(max(i,0),in.dim(0)-1),min(max(j,0),in.dim(1)-1)) < T) {
+                        isolated++;
+                    }
+                }
+            }
+            if (isolated == 1) {
+                out(i,j) = 255;
+            }
+        }
+    }
+}
+
 // _____________________________________________________________________________
 
 void FeatureExtractor::setImage(bytearray &image) {
-    copy(lineimage,image);        
+    have_line_info = false;
+    copy(lineimage,image);
 }
 
 void FeatureExtractor::setLineInfo(int basepoint_in,int xheight_in) {
+    have_line_info = true;
     basepoint = basepoint_in;
     xheight = xheight_in;
+    logger.format("basepoint: %d, xheight: %d", basepoint, xheight);
 }
 
 void FeatureExtractor::getFeatures(floatarray &feature,
@@ -658,6 +690,11 @@ void FeatureExtractor::getFeatures(floatarray &feature,
                                    FeatureType ftype) {
     bytearray subimage;
     crop(subimage,lineimage,r);
+    if (removeisolatedpixels) {
+        bytearray temp;
+        remove_isolated_pixels(temp, subimage, 0.2, 20);
+        copy(subimage,temp);
+    }
     if(ftype==IMAGE) {
         preprocess(subimage);
         calculate_image_feature(feature,subimage);
@@ -674,6 +711,10 @@ void FeatureExtractor::getFeatures(floatarray &feature,
         preprocess(subimage);
         calculate_skel_feature(feature,subimage);
     }
+    else if(ftype==SKEL2) {
+        preprocess(subimage);
+        calculate_skel_feature2(feature,subimage);
+    }
     else if(ftype==SKELPTS) {
         preprocess(subimage);
         calculate_skelpts_feature(feature,subimage);
@@ -688,6 +729,60 @@ void FeatureExtractor::getFeatures(floatarray &feature,
     else if(ftype==RELSIZE) {
         calculate_relsize_feature(feature,subimage);
     }
+    else if(ftype==HISTO) {
+        calculate_histogram_feature(feature,subimage);
+    }
+    else {
+        throw "feature type not implemented yet.";
+    }
+}
+
+void FeatureExtractor::getFeatures(floatarray &feature, bytearray
+                                   &input_image,FeatureType ftype) {
+    bytearray subimage;
+    if (removeisolatedpixels) {
+        remove_isolated_pixels(subimage, input_image, 0.2, 20);
+    } else {
+        copy(subimage,input_image);
+    }
+
+    if(ftype==IMAGE) {
+        preprocess(subimage);
+        calculate_image_feature(feature,subimage);
+    }
+    else if(ftype==GRAD) {
+        preprocess(subimage);
+        calculate_grad_feature(feature,subimage);
+    }
+    else if(ftype==BAYS) {
+        preprocess(subimage);
+        calculate_bays_feature(feature,subimage);
+    }
+    else if(ftype==SKEL) {
+        preprocess(subimage);
+        calculate_skel_feature(feature,subimage);
+    }
+    else if(ftype==SKEL2) {
+        preprocess(subimage);
+        calculate_skel_feature2(feature,subimage);
+    }
+    else if(ftype==SKELPTS) {
+        preprocess(subimage);
+        calculate_skelpts_feature(feature,subimage);
+    }
+    else if(ftype==INCL) {
+        preprocess(subimage);
+        calculate_incl_feature(feature,subimage);
+    }
+    else if(ftype==POS) {
+        calculate_pos_feature(feature,subimage);
+    }
+    else if(ftype==RELSIZE) {
+        calculate_relsize_feature(feature,subimage);
+    }
+    else if(ftype==HISTO) {
+        calculate_histogram_feature(feature,subimage);
+    }
     else {
         throw "feature type not implemented yet.";
     }
@@ -702,50 +797,6 @@ void FeatureExtractor::appendFeatures(floatarray &vector,rectangle r,
     }
 }
 
-void FeatureExtractor::appendFeaturesRescaled(floatarray &vector,int w,int h,
-                                              rectangle r,
-                                              FeatureType ftype) {
-}
-
-void FeatureExtractor::getFeatures(floatarray &feature, bytearray
-                                   &input_image,FeatureType ftype) {
-    bytearray subimage;
-    copy(subimage,input_image);
-    if(ftype==IMAGE) {
-        preprocess(subimage);
-        calculate_image_feature(feature,subimage);
-    }
-    else if(ftype==GRAD) {
-        preprocess(subimage);
-        calculate_grad_feature(feature,subimage);
-    }
-    else if(ftype==BAYS) {
-        preprocess(subimage);
-        calculate_bays_feature(feature,subimage);
-    }
-    else if(ftype==SKEL) {
-        preprocess(subimage);
-        calculate_skel_feature(feature,subimage);
-    }
-    else if(ftype==SKELPTS) {
-        preprocess(subimage);
-        calculate_skelpts_feature(feature,subimage);
-    }
-    else if(ftype==INCL) {
-        preprocess(subimage);
-        calculate_incl_feature(feature,subimage);
-    }
-    else if(ftype==POS) {
-        calculate_pos_feature(feature,subimage);
-    }
-    else if(ftype==RELSIZE) {
-        calculate_relsize_feature(feature,subimage);
-    }
-    else {
-        throw "feature type not implemented yet.";
-    }
-}
-
 void FeatureExtractor::appendFeatures(floatarray &vector,bytearray &subimage,
                                       FeatureType ftype) {
     floatarray feature;
@@ -755,6 +806,19 @@ void FeatureExtractor::appendFeatures(floatarray &vector,bytearray &subimage,
     }
 }
 
+void FeatureExtractor::appendFeaturesRescaled(floatarray &vector,int w,int h,
+                                              rectangle r,
+                                              FeatureType ftype) {
+}
+
+void fill_image(int &c, floatarray &feature, bytearray &img) {
+    for(int j=0;j<img.dim(1);++j) {
+        for(int i=0;i<img.dim(0);++i) {
+            feature(c++) = img(i,j);     /****/
+        }
+    }
+}
+
 void FeatureExtractor::calculate_image_feature(floatarray &image_feature,
                                                bytearray &input_image) {
     bytearray img;
@@ -769,11 +833,44 @@ void FeatureExtractor::calculate_image_feature(floatarray &image_feature,
     image_feature.resize(DIM_X*DIM_Y);
 
     int c = 0;
-    for(int j=0;j<rescaled.dim(1);++j) {
-        for(int i=0;i<rescaled.dim(0);++i) {
-            image_feature(c++) = rescaled(i,j);
+    fill_image(c, image_feature, rescaled);
+}
+
+void FeatureExtractor::calculate_histogram_feature(floatarray &image_feature,
+                                               bytearray &input_image) {
+    rectangle red_box = find_one_box(input_image,250);
+    rectangle out_box = scale_feature_box(red_box,DIM_X,DIM_Y);
+    bytearray rescaled;
+    rescaled.resize(DIM_X,DIM_Y);
+    fill(rescaled,255);
+    scale_box_gray_from_gray(rescaled,out_box,input_image,red_box);
+
+
+    floatarray h_histo;
+    floatarray v_histo;
+    h_histo.resize(DIM_X);
+    v_histo.resize(DIM_Y);
+    fill(h_histo, 0.0);
+    fill(v_histo, 0.0);
+
+    for(int i=0;i<rescaled.dim(0);++i) {
+        for(int j=0;j<rescaled.dim(1);++j) {
+            h_histo(i) += rescaled(i,j);
+            v_histo(j) += rescaled(i,j);
         }
     }
+
+    mul(h_histo, 1.f/DIM_Y);
+    mul(v_histo, 1.f/DIM_X);
+
+    image_feature.resize(DIM_X+DIM_Y);
+    int c = 0;
+    for(int i=0;i<DIM_X;++i) {
+        image_feature(c++) = h_histo(i);
+    }
+    for(int i=0;i<DIM_Y;++i) {
+        image_feature(c++) = v_histo(i);
+    }
 }
 
 void FeatureExtractor::calculate_grad_feature(floatarray &grad_feature,
@@ -930,6 +1027,12 @@ void FeatureExtractor::calculate_bays_feature(floatarray &bays_feature,
 
 void FeatureExtractor::calculate_skel_feature(floatarray &skel_feature,
                                               bytearray &input_image) {
+
+    // This version of calculate skel_feature can have 2 issues:
+    // 1: the image is first rescaled without keeping the original width/height ratio
+    //    Consequence: a capital i in arial is a big black square
+    // 2: if the feature image has quite large dimensions, the 'dist' can quickly be
+    //    greater than 255, black values instead of white values are set.
     bytearray image;
     image.resize(DIM_X,DIM_Y);
     rectangle red_box = find_one_box(input_image);
@@ -953,14 +1056,46 @@ void FeatureExtractor::calculate_skel_feature(floatarray &skel_feature,
         dist+=DIST_STEP;
     } while(change);
 
-    int c = 0;
     skel_feature.resize(DIM_X*DIM_Y);
-    for(int j=0;j<image.dim(1);j++) {
-        for(int i=0;i<image.dim(0);i++) {
-            skel_feature(c) = image(i,j);
-            c++;
+
+    int c = 0;
+    fill_image(c, skel_feature, image);
+}
+
+void FeatureExtractor::calculate_skel_feature2(floatarray &skel_feature,
+                                              bytearray &input_image) {
+    bytearray image;
+    image.resize(DIM_X,DIM_Y);
+    rectangle red_box       = find_one_box(input_image,253);
+    rectangle scaled_box    = scale_feature_box(red_box,DIM_X,DIM_Y);
+
+    fill(image,255);
+    scale_box_bin(image,scaled_box,input_image,red_box,200);
+
+    thinning_box(image,scaled_box);
+    //while (peel4(image) == true) { ; }
+
+    int DIST_STEP = 10;
+    bool change;
+    byte dist=0;
+    do {
+        change = false;
+        for(int i=0;i<DIM_X;i++) {
+            for(int j=0;j<DIM_Y;j++) {
+                if(get(image,i,j)==dist) {
+                    if(color_neighbors(image,whole_image(image),point(i,j),dist+DIST_STEP))
+                        change=true;
+                }
+            }
         }
-    }
+        dist+=DIST_STEP;
+    } while(change && (dist < 255-DIST_STEP));
+
+
+    skel_feature.resize(DIM_X*DIM_Y);
+
+    int c = 0;
+    fill_image(c, skel_feature, image);
 }
 
 void FeatureExtractor::calculate_skelpts_feature(floatarray &skelpts_feature,
@@ -1006,18 +1141,9 @@ void FeatureExtractor::calculate_skelpts_feature(floatarray &skelpts_feature,
 
     int c = 0;
     skelpts_feature.resize(2*DIM_X*DIM_Y);
-    for(int j=0;j<out.dim(1);j++) {
-        for(int i=0;i<out.dim(0);i++) {
-            skelpts_feature(c) = out(i,j);
-            c++;
-        }
-    }
-    for(int j=0;j<out2.dim(1);j++) {
-        for(int i=0;i<out2.dim(0);i++) {
-            skelpts_feature(c) = out2(i,j);
-            c++;
-        }
-    }
+
+    fill_image(c, skelpts_feature, out);
+    fill_image(c, skelpts_feature, out2);
 }
 
 void FeatureExtractor::calculate_incl_feature(floatarray &incl_feature,
@@ -1042,15 +1168,13 @@ void FeatureExtractor::calculate_incl_feature(floatarray &incl_feature,
 
     incl_feature.resize(DIM_X*DIM_Y);
     int c = 0;
-    for(int j=0;j<out.dim(1);++j) {
-        for(int i=0;i<out.dim(0);++i) {
-            incl_feature(c++) = out(i,j);
-        }
-    }
+    fill_image(c, incl_feature, out);
 }
 
 void FeatureExtractor::calculate_pos_feature(floatarray &position_feature,
                                              bytearray &input_image) {
+    ALWAYS_ASSERT(have_line_info);
+    CHECK_CONDITION(xheight>0);
     position_feature.resize(DIM_X);
 
     fill(position_feature,0);
@@ -1104,10 +1228,12 @@ void FeatureExtractor::calculate_pos_feature(floatarray &position_feature,
 
 void FeatureExtractor::calculate_relsize_feature(floatarray &relsize_feature,
                                                  bytearray &input_image) {
+    ALWAYS_ASSERT(have_line_info);
+    CHECK_CONDITION(xheight>0);
     rectangle box = find_one_box(input_image);
     float height = box.height();
     float rel_size = height/xheight;
-    
+
     // unary coding
     relsize_feature.resize(DIM_X);
     for (int j=0;j<relsize_feature.dim(0);++j) {
@@ -1119,8 +1245,32 @@ void FeatureExtractor::calculate_relsize_feature(floatarray &relsize_feature,
     gauss1d(relsize_feature,1.5);
 }
 
+void FeatureExtractor::feat2image(bytearray &image,floatarray &feature) {
+    // rotate+flip: image, inclusion, skeleton, skeleton points
+    int c=0;
+    int DIM_X_NFEAT = feature.length()/DIM_Y;
+    image.resize(DIM_X_NFEAT,DIM_Y);
+    for(int i=0;i<DIM_X_NFEAT;i++) {
+        for(int j=0;j<DIM_Y;j++) {
+            image(i,j) = byte(feature(c));
+            c++;
+        }
+    }
+}
+
 namespace ocropus {
     FeatureExtractor *make_FeatureExtractor() {
-        return new FeatureExtractor();
+        FeatureExtractor* FE = new FeatureExtractor();
+        FE->setFeatureSize(10,10);
+        FE->setRemoveIsolatedPixels(false);
+        FE->have_line_info = false;
+        return FE;
+    }
+    FeatureExtractor *make_FeatureExtractor(int x, int y) {
+        FeatureExtractor* FE = new FeatureExtractor();
+        FE->setFeatureSize(x,y);
+        FE->setRemoveIsolatedPixels(false);
+        FE->have_line_info = false;
+        return FE;
     }
 };
