@@ -16,10 +16,10 @@
 // limitations under the License.
 //
 // Project: bpnet -- Neural Network Classifier
-// File: bpnet.h
-// Purpose: Neural network classifier
+// File: grouping.h
+// Purpose: train and recognize lines
 // Responsible: Hagen Kaprykowsky (kapry@iupr.net)
-// Reviewer:
+// Reviewer: Yves Rangoni (rangoni@iupr.net)
 // Primary Repository:
 // Web Sites: www.iupr.org, www.dfki.de
 
@@ -29,7 +29,9 @@
 #include "colib.h"
 
 namespace ocropus {
-    colib::IRecognizeLine *make_NewGroupingLineOCR(colib::ICharacterClassifier *classifier, colib::ISegmentLine *segmenter, bool use_line_info = true);
+    colib::IRecognizeLine *make_NewGroupingLineOCR(colib::ICharacterClassifier *classifier, 
+        colib::ISegmentLine *segmenter, 
+        bool use_line_info = true);
 }
 
 #endif
