@@ -70,11 +70,11 @@ namespace ocropus {
         splittable = true;
     }
 
-    void CTextlineRASTBasic::set_max_slope(double max_slope){
+    void CTextlineRASTBasic::setMaxSlope(double max_slope){
         all_params[1] = interval(-max_slope,max_slope);
     }
 
-    void CTextlineRASTBasic::set_max_yintercept(double ymin, double ymax){
+    void CTextlineRASTBasic::setMaxYintercept(double ymin, double ymax){
         all_params[0] = interval(ymin,ymax);
     }
 
@@ -171,13 +171,13 @@ namespace ocropus {
         queue.insert(initial,initial->priority);
     }
     
-    void CTextlineRASTBasic::make_substates(narray<CState> &substates,CState &state) {
+    void CTextlineRASTBasic::makeSubStates(narray<CState> &substates,CState &state) {
         substates.clear();
         Parameters &p = state->params;
         
         int mi = -1;
         double mv = -1e30;
-        for(int i = 0;i<nparams;i++) {
+        for(int i = 0;i<ntlparams;i++) {
             double v = p[i].width()*splitscale[i];
             if(v>mv) { mi = i; mv = v; }
         }
@@ -194,7 +194,7 @@ namespace ocropus {
     }
 
     //Assuming horizontal lines with slope in the interval [-0.05, 0.05]
-    int CTextlineRASTBasic::wbox_intersection(CState &top){
+    int CTextlineRASTBasic::wboxIntersection(CState &top){
         
         Matches &matches = top->matches;
 
@@ -228,7 +228,7 @@ namespace ocropus {
             }
             if(use_whitespace){
                 Matches &matches = top->matches;
-                int index = wbox_intersection(top);
+                int index = wboxIntersection(top);
                 if(index >= 0){
                     Matches leftmatches,rightmatches;
                     for(int i = 0;i<matches.length();i++) {
@@ -270,11 +270,11 @@ namespace ocropus {
                 }
             }
             if(final(top->quality,top->params)) {
-                push_result(top);
+                pushResult(top);
                 continue;
             }
             narray<CState> substates;
-            make_substates(substates,top);
+            makeSubStates(substates,top);
             for(int i = 0;i<substates.length();i++) {
                 if(substates[i]->quality.hi<min_q) continue;
                 if(substates[i]->matches.length()<min_count) continue;
@@ -283,7 +283,7 @@ namespace ocropus {
         }
     }
     
-    void CTextlineRASTBasic::push_result(CState &result){
+    void CTextlineRASTBasic::pushResult(CState &result){
         Matches &matches = result->matches;
         for(int i = 0;i<matches.length();i++)
             used[matches[i]] = true;
@@ -299,7 +299,17 @@ namespace ocropus {
             return;
         linestats = make_CharStats(*charstats);
         for(int i = 0, l = charstats->char_boxes.length(); i<l; i++){
-            cboxes.push(charstats->char_boxes[i]);
+            // Only consider those char boxes that do not overlap with any
+            // obstacle, i.e. not contained in any obstacle
+            bool contained=false;
+            for(int j=0; j<columns.length(); j++){
+                if(charstats->char_boxes[i].fraction_covered_by(columns[j]) > 0.5){
+                    contained=true;
+                    break;
+                }
+            }
+            if(!contained)
+                cboxes.push(charstats->char_boxes[i]);
         }
         for(int i = 0, l = columns.length(); i<l; i++){
             wboxes.push(columns[i]);
@@ -332,6 +342,9 @@ namespace ocropus {
             textlines.push(results[i]->returnLineParam());
         }
     }
+    CTextlineRASTBasic *make_CTextlineRASTBasic(){
+        return new CTextlineRASTBasic();
+    }
 
     void CTextlineRAST::setDefaultParameters() {
         generation = 0;
@@ -375,7 +388,7 @@ namespace ocropus {
         setDefaultParameters();
     }
 
-    void CTextlineRAST::push_result(CState &result){
+    void CTextlineRAST::pushResult(CState &result){
         Matches &matches = result->matches;
         for(int i = 0;i<matches.length();i++)
             used[matches[i]] = true;
@@ -400,7 +413,10 @@ namespace ocropus {
         tl.bbox = rectangle();
         for(int i = 0;i<matches.length();i++){
             float box_height = cboxes[matches[i]].y1 - cboxes[matches[i]].y0;
-            if(box_height > min_box_height * xheight) continue;
+            if(box_height > min_box_height * xheight){
+                used[matches[i]] = false;
+                continue;
+            }
             tl.bbox.include(cboxes[matches[i]]);
         }
                 
@@ -448,6 +464,7 @@ namespace ocropus {
         }  
                 
         result_lines.push(tl);
+        results.push(result);
         generation++;
     }
     
@@ -464,8 +481,19 @@ namespace ocropus {
         result_lines.clear();
         cboxes.clear();
         cboxes_all.clear();
-        for(int i = 0, l = charstats->char_boxes.length(); i<l; i++)
-            cboxes.push(charstats->char_boxes[i]);
+        for(int i = 0, l = charstats->char_boxes.length(); i<l; i++){
+            // Only consider those char boxes that do not overlap with any
+            // obstacle, i.e. not contained in any obstacle
+            bool contained=false;
+            for(int j=0; j<columns.length(); j++){
+                if(charstats->char_boxes[i].fraction_covered_by(columns[j]) > 0.5){
+                    contained=true;
+                    break;
+                }
+            }
+            if(!contained)
+                cboxes.push(charstats->char_boxes[i]);
+        }
         used.resize(cboxes.length());
         fill(used,false);
         for(int i = 0, l = charstats->dot_boxes.length(); i<l; i++)
