@@ -26,8 +26,7 @@
 #include <time.h>
 #include "ocr-layout-rast.h"
 
-using namespace imgio;
-using namespace imglib;
+using namespace iulib;
 using namespace colib;
 
 namespace ocropus {
@@ -72,12 +71,15 @@ namespace ocropus {
         return ( (y > wbox.y0) && (y < wbox.y1) );
     }
     
+    // FIXME comment --tmb
+    
     static void extend_lines(narray<line> &lines, rectarray &wboxes,
                              int image_width){
 
         int num_lines = lines.length();
         int num_wboxes = wboxes.length();
         
+        // FIXME: extend to image start/width if not intersecting with column separator 
         for(int i = 0; i<num_lines; i++){
             float new_start = 0;
             float new_end = image_width-1;
@@ -95,6 +97,8 @@ namespace ocropus {
         }
     }
 
+    // FIXME put the graphics stuff into a separate source file --tmb
+
     static void paint_line(intarray &image, line l){
         int width=image.dim(0);
         int height=image.dim(1);
@@ -128,47 +132,7 @@ namespace ocropus {
         
     }
 
-    //Paint the given rectangle with yellow color.
-    //The color format is xRGB, which means the hexadecimal no. 0x00FF0000 is red.
-    static void paint_box(intarray &image, rectangle b, int color){
-        int width=image.dim(0);
-        int height=image.dim(1);
-        int left, top, right, bottom;
-        left   = (b.x0<0) ? 0 : b.x0;
-        top    = (b.y0<0) ? 0 : b.y0;
-        right  = (b.x1>=width) ? width-1 : b.x1;
-        bottom = (b.y1>=height) ? height-1 : b.y1;
-
-        if(right <= left || bottom <= top) return;
-
-        for(int x= left;x< right;x++){
-            for(int y= top;y< bottom;y++){
-                image(x,y)&=color;
-            }
-        }
-        
-    }
-
-    //Paint only the box border
-    static void paint_box_border(intarray &image, rectangle b, int color){
-        int width=image.dim(0);
-        int height=image.dim(1);
-        int left, top, right, bottom;
-       
-        left   = (b.x0<0)  ? 0   : b.x0;
-        top    = (b.y0<0)  ? 0   : b.y0;
-        right  = (b.x1>=width) ? width-1 : b.x1;
-        bottom = (b.y1>=height) ? height-1 : b.y1;
-
-        if(right <= left || bottom <= top) return;
-
-        int x,y;
-        for(x=left;x<=right;x++){ image(x,top)     &=color; }
-        for(x=left;x<=right;x++){ image(x,bottom)  &=color; }
-        for(y=top;y<=bottom;y++){ image(left,y)    &=color; }
-        for(y=top;y<=bottom;y++){ image(right,y)   &=color; }
-        
-    }
+    // FIXME explain what this does --tmb
 
     static void connect_line_centers(intarray &image, line a, line b){
         int width=image.dim(0);
@@ -222,8 +186,20 @@ namespace ocropus {
         }
     }
 
-    void SegmentPageByRAST::segmentInternal(intarray &visualization, intarray &image,bytearray &in_not_inverted, bool need_visualization) {
+    // FIXME refactor this
+    // make the arguments instance variables
+    // make separate private methods out of the individual processing steps
+    // give descriptive names to each such private method
+    // this documents the individual steps and makes the overall algorithm easier to follow
+    // --tmb
 
+    void SegmentPageByRAST::segmentInternal(intarray &visualization, 
+                                            intarray &image,
+                                            bytearray &in_not_inverted, 
+                                            bool need_visualization,
+                                            rectarray &extra_obstacles) {
+
+        // FIXME remove this dead code --tmb
         //float startTime = clock()/float(CLOCKS_PER_SEC);
         const int zero   = 0;
         const int yellow = 0x00ffff00;
@@ -248,8 +224,8 @@ namespace ocropus {
         }
         //fprintf(stderr,"Time elapsed (bounding_boxes): %.3f \n",(clock()/float(CLOCKS_PER_SEC)) - startTime);
         autodel<CharStats> charstats(make_CharStats());
-        charstats->get_char_boxes(bboxes);
-        charstats->calc_char_stats();
+        charstats->getCharBoxes(bboxes);
+        charstats->calcCharStats();
         if(debug_layout>=2){
             charstats->print();
         }
@@ -264,8 +240,16 @@ namespace ocropus {
         // Find column separators
         autodel<ColSeparators> gutters(make_ColSeparators());
         rectarray columns,colcandidates;
-        gutters->find_gutters(colcandidates,whitespaceboxes,*charstats);
-        gutters->filter_overlaps(columns,colcandidates);
+        // FIXME
+        // why does findGutters return a list of "colcandidates"?
+        // are colcandidates gutters?  or what?
+        // --tmb
+        gutters->findGutters(colcandidates,whitespaceboxes,*charstats);
+        gutters->filterOverlaps(columns,colcandidates);
+        // it appears that this isn't a list of columns, but a list of
+        // obstacles, so add the user-supplied obstacles as well
+        for(int i=0;i<extra_obstacles.length();i++)
+            columns.push(extra_obstacles[i]);
         if(debug_layout){
             for(int i=0; i<columns.length();i++){
                 printf("%d %d %d %d\n",columns[i].x0,columns[i].y0,
@@ -281,40 +265,49 @@ namespace ocropus {
         ctextline->min_count = 2;   // ---- number of characters in a textline
         ctextline->min_length= 30;  // ---- length in pixels of a textline
         ctextline->extract(textlines,columns,charstats);
-        rosort(textlines,columns,*charstats);
+        roSort(textlines,columns,*charstats);
         //fprintf(stderr,"Time elapsed (ctextline): %.3f \n",(clock()/float(CLOCKS_PER_SEC)) - startTime);
 
+        // FIXME paragraph extraction should work generically on segmentation images.
+        // It also doesn't belong inside the main RAST loop. --tmb
+        
         //rectarray paragraphs;
         rectarray textcolumns;
-        //ctextline->grouppara(paragraphs,textlines,charstats);
-        getcol(textcolumns,textlines,columns);
+        //ctextline->groupPara(paragraphs,textlines,charstats);
+        getCol(textcolumns,textlines,columns);
         color(image,in,textlines,textcolumns);
         //fprintf(stderr,"Time elapsed (find-columns): %.3f \n",(clock()/float(CLOCKS_PER_SEC)) - startTime);
-//         if(debug_layout){
-//             for(int i=0; i<textlines.length();i++)
-//                 textlines[i].print();
-//             for (int i=0; i<textcolumns.length();i++)
-//                 textcolumns[i].println();
-//         }
+        if(debug_layout){
+            for(int i=0; i<textlines.length();i++)
+                textlines[i].print();
+            for (int i=0; i<textcolumns.length();i++)
+                textcolumns[i].println();
+        }
         replace_values(image,zero,yellow);
         if(need_visualization) {
             visualizeLayout(visualization, in_not_inverted, textlines, columns, *charstats);
         }
     }
 
-    void SegmentPageByRAST::segment(intarray &result, bytearray &in_not_inverted) {
+    void SegmentPageByRAST::segment(intarray &result, bytearray &in_not_inverted,rectarray &obstacles) {
         intarray debug_image;
         if(debug_segm) {
-            segmentInternal(debug_image, result, in_not_inverted, true);
-            write_png_rgb(stdio(debug_segm,"w"), debug_image);
+            segmentInternal(debug_image, result, in_not_inverted, true, obstacles);
+            write_image_packed(debug_segm,debug_image);
         } else {
-            segmentInternal(debug_image, result, in_not_inverted, false);
+            segmentInternal(debug_image, result, in_not_inverted, false, obstacles);
         }
     }
 
+    void SegmentPageByRAST::segment(intarray &result, bytearray &in_not_inverted) {
+        rectarray obstacles;
+        segment(result,in_not_inverted,obstacles);
+    }
+
     void SegmentPageByRAST::visualize(intarray &result, bytearray &in_not_inverted) {
         intarray segmentation;
-        segmentInternal(result, segmentation, in_not_inverted, true);
+        rectarray obstacles;
+        segmentInternal(result, segmentation, in_not_inverted, true,obstacles);
     }
 
     void SegmentPageByRAST::visualizeLayout(intarray &debug_image,
@@ -406,7 +399,7 @@ namespace ocropus {
         ro_index.push(k);
     }
     
-    void SegmentPageByRAST::depth_first_search(narray<bool> &lines_dag){
+    void SegmentPageByRAST::depthFirstSearch(narray<bool> &lines_dag){
         //void visit (int k);
         int size = lines_dag.dim(0);
         val.resize(size);
@@ -416,7 +409,9 @@ namespace ocropus {
                 visit(k, lines_dag);
     }
     
-    void SegmentPageByRAST::rosort(narray<TextLine> &textlines,
+    // FIXME use descriptive name; "ro"??? --tmb
+
+    void SegmentPageByRAST::roSort(narray<TextLine> &textlines,
                                    rectarray &columns,
                                    CharStats &charstats){
         id = 0;
@@ -440,7 +435,7 @@ namespace ocropus {
         fill(lines_dag,false);
         
         construct_graph(lines, lines_dag);
-        depth_first_search(lines_dag);
+        depthFirstSearch(lines_dag);
         int size = ro_index.length();
         //"\nNumber of connected lines = " << size <<"\n";
         textlines.clear();
@@ -449,11 +444,15 @@ namespace ocropus {
         }
     }
     
+    // FIXME newColumn --tmb
+
     static bool new_column(rectangle current, rectangle previous){
         if(current.y0 > previous.y1){return true;}
         return false;
     }
     
+    // FIXME name getBoundingBox --tmb
+    
     static void getbbox(rectangle &bbox,rectarray &bboxes){
         //first copying into arrays for x0,x1,y0,y1
         bbox = rectangle();
@@ -470,6 +469,8 @@ namespace ocropus {
         return NOT_ALIGNED;
     }
 
+    // FIXME name getRelativeAlignment (?) --tmb
+    
     static int getrelalign(rectangle &current,rectangle &previous){
         bool left = false;
         bool right = false;
@@ -489,6 +490,8 @@ namespace ocropus {
         return alignment(left, right, center);
     }
 
+    // FIXME name getAlignment (?) or something else (?) --tmb
+
     static void getalign(objlist< narray <int> > &tempalign, 
                          objlist< narray <rectangle> > &temppara){
         narray<int> talign;
@@ -540,17 +543,28 @@ namespace ocropus {
         }
     }
 
+    // FIXME name matchesPrevious --tmb
+
     static bool matches_previous(int previous_alignment, int tempalign){
         return (previous_alignment == 0 && tempalign != 0) ||
             (previous_alignment != 0 && tempalign == previous_alignment);
     }
       
+    // FIXME name doesNotMatchPrevious --tmb
+
     static bool not_matches_previous(int previous_alignment, int tempalign){
         return (previous_alignment == 0 && tempalign == 0) ||
             (previous_alignment != 0 && tempalign != previous_alignment);
     }
       
-    void SegmentPageByRAST::grouppara(rectarray &paragraphs ,
+    // FIXME
+    // name: groupParagraphs
+    // factor paragraph detection out of main RAST class
+    // ideally, make paragraph detection work for arbitrary color segmentation
+    // method too long
+    // --tmb
+
+    void SegmentPageByRAST::groupPara(rectarray &paragraphs ,
                                       narray<TextLine> &textlines,
                                       CharStats &charstats){
 
@@ -627,6 +641,8 @@ namespace ocropus {
         }
     }
     
+    // FIXME name getOverlap --tmb
+
     static float getoverlap(rectangle current,rectangle previous){
         int   intersection = (min(current.x1,previous.x1)-max(current.x0,previous.x0));
         float width_sum = (current.x1-current.x0)+(previous.x1-previous.x0);
@@ -636,10 +652,13 @@ namespace ocropus {
             return 0;
     }
     
+    // FIXME
+    // name getColumn (?) or something more descriptive
+    // method too long
+    // --tmb
     
     
-    
-    void SegmentPageByRAST::getcol(rectarray &columns,
+    void SegmentPageByRAST::getCol(rectarray &columns,
                                    rectarray &paragraphs){
         objlist< rectarray > tempcol;
         objlist< narray<float> > floatcol;
@@ -697,7 +716,13 @@ namespace ocropus {
         fclose(colfile);
     }
 
-    void SegmentPageByRAST::getcol(rectarray &textcolumns,
+    // FIXME
+    // method too long
+    // name: getColumn or more descriptive
+    // functionality overlaps with other overloaded method?
+    // --tmb
+
+    void SegmentPageByRAST::getCol(rectarray &textcolumns,
                                    narray<TextLine> &textlines,
                                    rectarray &gutters){
 
@@ -743,6 +768,12 @@ namespace ocropus {
         }
     }
 
+
+    // FIXME
+    // refactor and move visualization methods out of the main RAST class
+    // ideally, let visualization methods work on arbitrary inputs
+    // --tmb
+
     void SegmentPageByRAST::color(intarray &image,bytearray &in, 
                                   narray<TextLine> &textlines,
                                   rectarray &textcolumns){
@@ -765,8 +796,9 @@ namespace ocropus {
         color = 1;
         // Limit the number of text columns to what can actually be encoded (30)
         int MAX_COLS = 30;
+        ASSERTWARN(textcolumns.length()<=MAX_COLS);
         int num_cols = (textcolumns.length() < MAX_COLS)?textcolumns.length():MAX_COLS;
-        if(num_cols!=textcolumns.length()) {
+        if(num_cols!=textcolumns.length() && debug_layout) {
             fprintf(stderr,"\nWarning: Too many text columns: %d\n", textcolumns.length());
             fprintf(stderr,"         Max # of text columns: %d\n\n", MAX_COLS);
         }
