@@ -34,13 +34,12 @@
 #include "log-reg-data.h"
 
 using namespace ocropus;
-using namespace imgio;
-using namespace imglib;
+using namespace iulib;
 using namespace colib;
 
 namespace ocropus {
 
-    void ZoneFeatures::compress_hist(intarray &histogram){
+    void ZoneFeatures::compressHist(intarray &histogram){
         intarray histogram_comp;
         histogram_comp.resize(MAX_LEN);
         fill(histogram_comp,0);
@@ -67,7 +66,7 @@ namespace ocropus {
             histogram.push(histogram_comp(i)); 
     }
 
-    void ZoneFeatures::compress_2dhist(intarray &histogram_2D){
+    void ZoneFeatures::compress2DHist(intarray &histogram_2D){
 
         int steps[2];
     
@@ -131,7 +130,7 @@ namespace ocropus {
     
 
 
-    void ZoneFeatures::horizontal_run_lengths(floatarray &resulthist,
+    void ZoneFeatures::horizontalRunLengths(floatarray &resulthist,
                                               floatarray &resultstats,
                                               const bytearray &image){
         
@@ -192,8 +191,8 @@ namespace ocropus {
 
             }
         }
-        compress_hist(histogram_fg);
-        compress_hist(histogram_bg);
+        compressHist(histogram_fg);
+        compressHist(histogram_bg);
         for(int i=0, l=histogram_fg.length(); i<l; i++)
             resulthist.push(histogram_fg[i]);
         for(int i=0, l=histogram_bg.length(); i<l; i++)
@@ -223,7 +222,7 @@ namespace ocropus {
         
     }
 
-    void ZoneFeatures::vertical_run_lengths(floatarray &resulthist,
+    void ZoneFeatures::verticalRunLengths(floatarray &resulthist,
                                             floatarray &resultstats,
                                             const bytearray &image){
         
@@ -284,8 +283,8 @@ namespace ocropus {
 
             }
         }
-        compress_hist(histogram_fg);
-        compress_hist(histogram_bg);
+        compressHist(histogram_fg);
+        compressHist(histogram_bg);
         for(int i=0, l=histogram_fg.length(); i<l; i++)
             resulthist.push(histogram_fg[i]);
         for(int i=0, l=histogram_bg.length(); i<l; i++)
@@ -315,7 +314,7 @@ namespace ocropus {
         
     }
 
-    void ZoneFeatures::main_diag_run_lengths(floatarray &resulthist,
+    void ZoneFeatures::mainDiagRunLengths(floatarray &resulthist,
                                              floatarray &resultstats,
                                              const bytearray &image){
         
@@ -394,8 +393,8 @@ namespace ocropus {
         }
 
 
-        compress_hist(histogram_fg);
-        compress_hist(histogram_bg);
+        compressHist(histogram_fg);
+        compressHist(histogram_bg);
         for(int i=0, l=histogram_fg.length(); i<l; i++)
             resulthist.push(histogram_fg[i]);
         for(int i=0, l=histogram_bg.length(); i<l; i++)
@@ -425,7 +424,7 @@ namespace ocropus {
         
     }
 
-    void ZoneFeatures::side_diag_run_lengths(floatarray &resulthist,
+    void ZoneFeatures::sideDiagRunLengths(floatarray &resulthist,
                                              floatarray &resultstats,
                                              const bytearray &image){
         
@@ -504,8 +503,8 @@ namespace ocropus {
         }
 
 
-        compress_hist(histogram_fg);
-        compress_hist(histogram_bg);
+        compressHist(histogram_fg);
+        compressHist(histogram_bg);
         for(int i=0, l=histogram_fg.length(); i<l; i++)
             resulthist.push(histogram_fg[i]);
         for(int i=0, l=histogram_bg.length(); i<l; i++)
@@ -535,7 +534,7 @@ namespace ocropus {
         
     }
 
-    void ZoneFeatures::concomp_hist(floatarray &result, 
+    void ZoneFeatures::concompHist(floatarray &result, 
                                     rectarray &concomps){
         
         intarray histogram_width;
@@ -561,9 +560,9 @@ namespace ocropus {
             histogram_2D(min(width, MAX_LEN)-1,min(height, MAX_LEN)-1)++;
         }
 
-        compress_hist(histogram_width);
-        compress_hist(histogram_height);
-        compress_2dhist(histogram_2D);
+        compressHist(histogram_width);
+        compressHist(histogram_height);
+        compress2DHist(histogram_2D);
 
         for(int i=0, l=histogram_width.length(); i<l; i++)
             result.push(histogram_width[i]);
@@ -575,11 +574,11 @@ namespace ocropus {
 
     }
 
-    static inline double distance (float x1, float y1, float x2, float y2) {
+    static inline double distance(float x1, float y1, float x2, float y2) {
         return (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1);
     }
 
-    void ZoneFeatures::concomp_neighbors(floatarray &result, 
+    void ZoneFeatures::concompNeighbors(floatarray &result, 
                                          rectarray &concomps){
         // bounding boxes nearest neighbor distance
         int num_boxes = concomps.length();
@@ -604,13 +603,13 @@ namespace ocropus {
             histogram( int (dist_min) )++;
         }
 
-        compress_hist(histogram);
+        compressHist(histogram);
         for(int i=0, l=histogram.length(); i<l; i++)
             result.push(histogram[i]);
 
     }
 
-    void ZoneFeatures::extract_features(floatarray &feature, bytearray &image){
+    void ZoneFeatures::extractFeatures(floatarray &feature, bytearray &image){
         if(!contains_only(image, byte(0), byte(255))){
             fprintf(stderr,"Binary image expected! ");
             fprintf(stderr,"skipping feature extraction...\n");
@@ -619,10 +618,10 @@ namespace ocropus {
 
         // RUNNING LENGTHS
         floatarray rl_stats;
-        horizontal_run_lengths(feature,rl_stats,image);
-        vertical_run_lengths(feature,rl_stats,image);
-        main_diag_run_lengths(feature,rl_stats,image);
-        side_diag_run_lengths(feature,rl_stats,image);
+        horizontalRunLengths(feature,rl_stats,image);
+        verticalRunLengths(feature,rl_stats,image);
+        mainDiagRunLengths(feature,rl_stats,image);
+        sideDiagRunLengths(feature,rl_stats,image);
 
         for(int index=0; index<rl_stats.length(); index++)
             feature.push(rl_stats[index]);
@@ -630,7 +629,9 @@ namespace ocropus {
         // CONNECTED COMPONENTS
         bytearray in;
         copy(in, image);
-        make_page_binary_and_black(in);
+        binarize_simple(in);
+        optional_check_background_is_lighter(in);
+        invert(in);
     
         // Do connected component analysis
         intarray charimage;
@@ -644,8 +645,8 @@ namespace ocropus {
             if(bboxes[i].area())
                 boxes.push(bboxes[i]);
 
-        concomp_hist(feature, boxes);
-        concomp_neighbors(feature, boxes);
+        concompHist(feature, boxes);
+        concompNeighbors(feature, boxes);
         
         int z = 1; 
         feature.push(z);
@@ -656,7 +657,7 @@ namespace ocropus {
         return new ZoneFeatures();
     }
     
-    void LogReg::load_data(){
+    void LogReg::loadData(){
         class_num = log_reg_class_num;
         factor = log_reg_factor;
         offset = log_reg_offset;
@@ -706,6 +707,27 @@ namespace ocropus {
         }
     }
 
+    void LogReg::getClassProbabilities(floatarray &probability,
+                                       floatarray &feature){
+        float sum,sum_total = 0;
+
+        probability.resize(class_num);
+
+        for(int k = 0; k < class_num; k++){
+            sum = 0;
+            for(int j = 0; j < feature_len; j++)                
+                sum += lambda(k,j) * feature(j);
+            sum = exp(factor * sum + feature_len * offset);
+
+            probability[k] = sum;
+            sum_total += sum;
+        }
+        
+        for(int k = 0; k < class_num; k++)
+            probability[k] /= sum_total;
+        
+    }
+
     LogReg *make_LogReg() {
         return new LogReg();
     }
