@@ -46,6 +46,12 @@
 #include "baseapi.h"
 #include "globals.h"
 
+#ifdef GOOGLE_INTERNAL
+#include "ccutil.h"
+#include "unicharset.h"
+using namespace tesseract;
+#endif
+
 // these Tess-defined macros interfere with IUPR names
 #undef rectangle
 #undef min
@@ -61,11 +67,11 @@
 #include "ocr-utils.h"
 #include "line-info.h"
 #include "segmentation.h"
+#include "logger.h"
 #include "ocr-segmentations.h"
 
 using namespace colib;
-using namespace imgio;
-using namespace imglib;
+using namespace iulib;
 using namespace ocropus;
 
 namespace ocropus {
@@ -75,6 +81,8 @@ namespace ocropus {
 extern BOOL_VAR_H(textord_ocropus_mode, FALSE, "Make baselines for ocropus");
 
 namespace {
+    Logger log_tesseract("tesseract");
+    Logger log_split("tesseract.split");
     static int area(rectangle rect) {
         if (rect.width() <= 0 || rect.height() <= 0)
             return 0;
@@ -83,7 +91,7 @@ namespace {
 
     int arg_max_overlap(narray<rectangle> &line_boxes, rectangle char_box) {
         int best_so_far = -1;
-        int best_overlap = -1;
+        int best_overlap = 0;
         for(int i = 0; i < line_boxes.length(); i++) {
             int overlap = area(line_boxes[i].intersection(char_box));
             if(overlap > best_overlap) {
@@ -109,48 +117,6 @@ namespace {
         return 2 * area(common) > area(inner);
     }*/
 
-#if 0
-    // Given an output of SimpleOCR (string, costs, bboxes) and the segmentation,
-    // produce a SegmentedOCR results (result, components).
-    //
-    // This uses heuristical match between bounding boxes returned by OCR and
-    // the segmentation bounding boxes. That might fail, especially in the case
-    // of undersegmentation.
-    static void convert_results(IGenericFst &result,
-                                idmap &components,
-                                nustring &string,
-                                floatarray &costs,
-                                narray<rectangle> &bboxes,
-                                intarray &segmentation) {
-        check_line_segmentation(segmentation);
-        make_line_segmentation_black(segmentation);
-        set_line_number(segmentation, 0);
-        components.clear();
-        int n = string.length();
-        ASSERT(costs.length() == n);
-        ASSERT(bboxes.length() == n);
-
-        narray<rectangle> seg_boxes;
-        bounding_boxes(seg_boxes, segmentation);
-
-        intarray ids(n);
-        for(int i=0; i<n; i++) { 
-            ids[i] = (string[i].ord() == ' '  ?  0  :  i + 1);
-        }
-
-        result.clear();
-        result.setString(string, costs, ids);
-        for(int i=0; i<n; i++) {
-            // fill components
-            for(int j=1; j<seg_boxes.length(); j++) {
-                if(fits_into(seg_boxes[j], bboxes[i])) {
-                    components.associate(ids[i], j);
-                }
-            }
-        }
-    }
-#endif
-
     template<class T>
     void fill_rectangle(narray<T> &a, rectangle r, T value) {
         r.intersect(rectangle(0, 0, a.dim(0), a.dim(1)));
@@ -177,10 +143,24 @@ namespace {
         }
         lattice.setString(text, costs, ids);
     }
+
+    rectangle unite_rectangles(narray<rectangle> &rects) {
+        rectangle result;
+        for(int i = 0; i < rects.length(); i++) {
+            if(!rects[i].empty())
+                result.include(rects[i]);
+        }
+        return result;
+    }
 };
 
 
 namespace ocropus {
+    colib::rectangle RecognizedPage::bbox(int index) {
+        return unite_rectangles(line_bboxes[index]);
+    }
+    
+
     enum {MIN_HEIGHT = 30};
 
     class TesseractWrapper;
@@ -255,16 +235,26 @@ static struct sigaction SIGSEGV_old;
     static int counter = 0;
     inline double sqr(double x) {return x * x;}
     class TesseractWrapper : TessBaseAPI /* , public ISimpleLineOCR */{
+        // tesseract_ is an instance of Tesseract; use that for everything
         int pass;
+        bytearray pixels;
 
         void pass_grayscale_image_to_tesseract(bytearray &image) {
-            unsigned char *pixels = new unsigned char[image.length1d()];
+            CHECK_ARG2(image.dim(0)>0 && image.dim(1)>0,"must provide non-empty image to Tesseract");
+            int n = image.length1d();
+            pixels.resize(n);
             for(int x=0;x<image.dim(0);x++) for(int y=0;y<image.dim(1);y++)
                 pixels[(image.dim(1) - y - 1) * image.dim(0) + x] = image(x,y);
 
-            CopyImageToTesseract(pixels, 1, image.dim(0), 0, 0, image.dim(0), image.dim(1));
-
-            delete [] pixels;
+#ifdef GOOGLE_INTERNAL
+            // SetImage does not copy the pixels
+            SetImage(&pixels[0],image.dim(0),image.dim(1),1,image.dim(0));
+            SetRectangle(0,0,image.dim(0),image.dim(1));
+#else
+            CopyImageToTesseract(&pixels[0], 1, image.dim(0), 0, 0, image.dim(0), image.dim(1));
+            // CopyImageToTesseract copies the pixels
+            pixels.resize(0);
+#endif
         }
 
 
@@ -296,10 +286,10 @@ static struct sigaction SIGSEGV_old;
                 for(int j = 0; j < multichar.length(); j++) {
                     str.push(multichar[j]);
                     rectangle &bbox = bboxes.push();
-                    bbox.x0 = x0[i];
-                    bbox.y0 = y0[i];
-                    bbox.x1 = x1[i];
-                    bbox.y1 = y1[i];
+                    bbox.x0 = x0[i] - 1; // don't ask me why -1; 
+                    bbox.y0 = y0[i] - 1; // I don't remember any padding --IM
+                    bbox.x1 = x1[i] - 1;
+                    bbox.y1 = y1[i] - 1;
                     costs.push(tess_costs[i] / multichar.length());
                 }
             }
@@ -319,7 +309,12 @@ static struct sigaction SIGSEGV_old;
             text.push(nuchar(truth));
             char buf[20];
             text.utf8Encode(buf, sizeof(buf));
-            if(!unicharset.contains_unichar(buf)) {
+#ifdef GOOGLE_INTERNAL
+            if(!tesseract_->unicharset.contains_unichar(buf))
+#else
+            if(!unicharset.contains_unichar(buf))
+#endif
+            {
                 //printf("Ouch! Character %s (%d) isn't known!\n", buf, truth);
                 return;
             }
@@ -337,7 +332,13 @@ static struct sigaction SIGSEGV_old;
 
         TesseractWrapper(const char *path_to_us) : pass(1) {
             if (!counter) {
+#ifdef GOOGLE_INTERNAL
+                if(!getenv("TESSDATA_PREFIX"))
+                    throw "must set TESSDATA_PREFIX in the environment";
+                Init(path_to_us, tesslanguage);
+#else
                 InitWithLanguage(path_to_us, NULL, tesslanguage, NULL, false, 0, NULL);
+#endif
 
                 // doesn't seem to do anything any longer
                 //textord_ocropus_mode.set_value(true); 
@@ -366,9 +367,16 @@ static struct sigaction SIGSEGV_old;
             enum {PADDING = 3};
             pad_by(image, PADDING, PADDING, colib::byte(255));
             pass_grayscale_image_to_tesseract(image);
+#ifdef GOOGLE_INTERNAL
+            ClearResults();
+            Threshold();
+            FindLines();
+            BLOCK_LIST &blocks = *block_list_;
+#else
             BLOCK_LIST blocks;
             FindLines(&blocks);
-
+#endif
+            if(getenv("TESS_DUMP")) DumpPGM(getenv("TESS_DUMP"));
         
             narray<autodel<WERD> > bln_words;
 
@@ -384,7 +392,11 @@ static struct sigaction SIGSEGV_old;
                                                &word->denorm);
                 bln_words.push() = bln_word;
                 BLOB_CHOICE_LIST_CLIST blob_choices;
-
+#ifdef GOOGLE_INTERNAL
+#define tess_default_matcher &Tesseract::tess_default_matcher
+#define tess_segment_pass1 tesseract_->tess_segment_pass1
+#define tess_segment_pass2 tesseract_->tess_segment_pass2
+#endif
                 if (pass == 1) {
                     word->best_choice = tess_segment_pass1(bln_word, &word->denorm,
                                                             tess_default_matcher,
@@ -397,6 +409,11 @@ static struct sigaction SIGSEGV_old;
                                                             word->outword);
                 }
 
+#ifdef GOOGLE_INTERNAL
+#undef tess_default_matcher
+#undef tess_segment_pass1
+#undef tess_segment_pass2
+#endif
                 //classify_word_pass1 (page_res_it.word(), page_res_it.row()->row, 
                 //FALSE, NULL, NULL);
 
@@ -543,11 +560,12 @@ static struct sigaction SIGSEGV_old;
                     e.extract(block_image, gray, i, /* margin: */ 1);
                     bytearray mask;
                     e.mask(mask, i, /* margin: */ 1);
-                    make_background_white(block_image);
+                    log_tesseract("pass 1 block before cleaning", block_image);
+                    optional_check_background_is_lighter(block_image);
                     
                     bytearray dilated_mask;
                     copy(dilated_mask, mask);
-                    dilate_circle(dilated_mask, 3);
+                    binary_dilate_circle(dilated_mask, 3);
 
                     ASSERT(samedims(mask, block_image));
                     ASSERT(samedims(dilated_mask, block_image));
@@ -559,6 +577,7 @@ static struct sigaction SIGSEGV_old;
                     e.mask(block_image, i, /* margin: */ 1);
                     invert(block_image);
                 }
+                log_tesseract("pass 1 block after cleaning", block_image);
                 pass_grayscale_image_to_tesseract(block_image);
                 block_lists[i-1] = TessBaseAPI::FindLinesCreateBlockList();
                 block_results[i-1] = TessBaseAPI::RecognitionPass1(block_lists[i-1]);
@@ -578,6 +597,7 @@ static struct sigaction SIGSEGV_old;
                     e.mask(block_image, i, /* margin: */ 1);
                     invert(block_image);
                 }
+                log_tesseract("pass 2 block", block_image);
                 pass_grayscale_image_to_tesseract(block_image);
                 block_results[i-1] = TessBaseAPI::RecognitionPass2(block_lists[i-1], block_results[i-1]);
                 extract_result_from_PAGE_RES(text[i-1],
@@ -631,7 +651,11 @@ static struct sigaction SIGSEGV_old;
                         whole_zone_bboxes[zone].y0 + zone_bboxes[zone][i].y0,
                         whole_zone_bboxes[zone].x0 + zone_bboxes[zone][i].x1,
                         whole_zone_bboxes[zone].y0 + zone_bboxes[zone][i].y1);
+                    log_split("char rectangle (page coords)", abs_charbox);
                     line = arg_max_overlap(line_bboxes, abs_charbox);
+                    log_split.format("char %d (%lc) gets into line %d", i, 
+                                     zone_text[zone][i].ord(),
+                                     line);
                     if(line == -1) continue;
                     if(pending_space[line]) {
                         text[line].push(nuchar(' '));
@@ -683,7 +707,7 @@ static struct sigaction SIGSEGV_old;
         }
 
         TesseractRecognizeLine() {
-            tess = new TesseractWrapper("");
+            tess = new TesseractWrapper(0);
             training = false;
         }
 
@@ -706,7 +730,8 @@ static struct sigaction SIGSEGV_old;
             // crude binarization (FIXME?) ...
             bytearray binarized;
             binarize_simple(binarized, image);
-            make_background_white(binarized);
+            optional_check_background_is_lighter(binarized);
+
             for(int i = 0; i < segmentation.length1d(); i++) {
                 if(binarized.at1d(i))
                     segmentation.at1d(i) = 0;
@@ -720,15 +745,7 @@ static struct sigaction SIGSEGV_old;
         return new TesseractRecognizeLine();
     }
 }
-namespace {
-    rectangle unite_rectangles(narray<rectangle> &rects) {
-        rectangle result;
-        for(int i = 0; i < rects.length(); i++) {
-            result.include(rects[i]);
-        }
-        return result;
-    }
-}
+
 namespace ocropus {
     void tesseract_recognize_blockwise(RecognizedPage &result, colib::bytearray &gray, colib::intarray &pageseg) {
         //double start = now();
@@ -738,29 +755,58 @@ namespace ocropus {
         tesseract_recognize_blockwise_and_split_to_lines(text, bboxes, costs, gray, pageseg);
         result.setWidth(gray.dim(0));
         result.setHeight(gray.dim(1));
-        result.setLinesCount(text.length());
+        int nonempty_count = 0;
         for(int i = 0; i < text.length(); i++) {
-            result.setText(text[i], i);
-            result.setBbox(unite_rectangles(bboxes[i]), i);
-            result.setCosts(costs[i], i);
+            if(text[i].length())
+                nonempty_count++;
+        }
+
+        result.setLinesCount(nonempty_count);
+        int j = 0;
+        for(int i = 0; i < text.length(); i++) {
+            if(!text[i].length())
+                continue;
+            CHECK_CONDITION(bboxes.length() == text.length());
+            result.setText(text[i], j);
+            result.setBboxes(bboxes[i], j);
+            result.setCosts(costs[i], j);
+            j++;
         }
         // description
         // Time report        
     }
+
+#ifdef GOOGLE_INTERNAL
+    bytearray tess_pixels;
+    autodel<TessBaseAPI> tess_instance;
+#endif
+
     void tesseract_init_with_language(const char *language) {
+#ifdef GOOGLE_INTERNAL
+        tess_instance = new TessBaseAPI();
+        if(!getenv("TESSDATA_PREFIX"))
+            throw "you must set TESSDATA_PREFIX when running in the Google environment";
+        tess_instance->Init(0,language);
+#else
         TessBaseAPI::InitWithLanguage(0,0,language,0,false,0,0);
+#endif
     }
 
     char *tesseract_rectangle(bytearray &image,int x0,int y0,int x1,int y1) {
+        CHECK_ARG(0<=x0 && x0<x1 && x1<=image.dim(0));
+        CHECK_ARG(0<=y0 && y0<y1 && y1<=image.dim(1));
+#ifdef GOOGLE_INTERNAL
+        math2raster(tess_pixels,image);
+        tess_instance->SetImage(&tess_pixels(0,0),tess_pixels.dim(1),tess_pixels.dim(0),
+                                1,tess_pixels.dim(1));
+        tess_instance->SetRectangle(x0,y0,x1,y1);
+        char *text = tess_instance->GetUTF8Text();
+        if(getenv("TESS_DUMP")) tess_instance->DumpPGM(getenv("TESS_DUMP"));
+#else
         bytearray temp;
-        ASSERT(0 <= x0);
-        ASSERT(x0 < x1);
-        ASSERT(x1 <= image.dim(0));
-        ASSERT(0 <= y0);
-        ASSERT(y0 < y1);
-        ASSERT(y1 <= image.dim(1));
         math2raster(temp,image);
         char *text = TessBaseAPI::TesseractRect(&temp(0,0),1,temp.dim(1),x0,y0,x1,y1);
+#endif
         return text;
     }
 
@@ -769,7 +815,11 @@ namespace ocropus {
     }
 
     void tesseract_end() {
+#ifdef GOOGLE_INTERNAL
+        tess_instance->End();
+        tess_instance = 0;
+#else
         TessBaseAPI::End();
+#endif
     }
-
 }
