@@ -1,3 +1,5 @@
+#ifndef didegrade_h__
+#define didegrade_h__
 // Copyright 2007 Deutsches Forschungszentrum fuer Kuenstliche Intelligenz
 // or its licensors, as applicable.
 // 
@@ -36,3 +38,4 @@ namespace ocropus {
                  double threshold_mean = .4,
                  double threshold_sigma = .04);
 };
+#endif
