@@ -31,11 +31,11 @@
 #include "sysutil.h"
 #include "ocr-segmentations.h"
 
-using namespace imgio;
-using namespace imglib;
+using namespace iulib;
 using namespace colib;
 
 namespace ocropus {
+    param_bool bgcheck("bgcheck", true, "abort on detecting an inverted image");
 
     void invert(bytearray &a) {
         int n = a.length1d();
@@ -77,12 +77,16 @@ namespace ocropus {
             sum += a(0, y);
         for(int y = 1; y < top; y++)
             sum += a(right, y);
+        // If average border intensity is between 127-128, inverting the
+        // image does not work correctly
+        float average_border_intensity = sum / ((right + top) * 2.0);
+        ASSERTWARN(average_border_intensity<=127 || average_border_intensity>=128);
         return sum / ((right + top) * 2);
     }
 
-    
+
     // FIXME use imgmorph stuff
-    
+
     void blit2d(bytearray &dest, const bytearray &src, int shift_x, int shift_y) {
         int w = src.dim(0);
         int h = src.dim(1);
@@ -175,13 +179,13 @@ namespace ocropus {
         write_png(stream,image);
         fclose(stream);
     }
-    
-    void paint_box(intarray &image, rectangle r, int color){
+
+    void paint_box(intarray &image, rectangle r, int color, bool inverted){
 
         int width  = image.dim(0);
         int height = image.dim(1);
         int left, top, right, bottom;
-       
+
         left   = (r.x0<0)  ? 0   : r.x0;
         top    = (r.y0<0)  ? 0   : r.y0;
         right  = (r.x1>=width) ? width-1 : r.x1;
@@ -191,28 +195,85 @@ namespace ocropus {
 
         for(int x=left; x<right; x++){
             for(int y=top; y<bottom; y++){
-                image(x,y) &= color;  
+                if(!inverted)
+                    image(x,y) &= color;
+                else
+                    image(x,y) |= color;
             }
         }
     }
 
-    void paint_box_border(intarray &image, rectangle r, int color){
+    void paint_box(bytearray &image, rectangle r, byte color, bool inverted){
 
         int width  = image.dim(0);
         int height = image.dim(1);
         int left, top, right, bottom;
-       
+
+        left   = (r.x0<0)  ? 0   : r.x0;
+        top    = (r.y0<0)  ? 0   : r.y0;
+        right  = (r.x1>=width) ? width-1 : r.x1;
+        bottom = (r.y1>=height) ? height-1 : r.y1;
+
+        if(right <= left || bottom <= top) return;
+
+        for(int x=left; x<right; x++){
+            for(int y=top; y<bottom; y++){
+                if(!inverted)
+                    image(x,y) &= color;
+                else
+                    image(x,y) |= color;
+            }
+        }
+    }
+
+    void paint_box_border(intarray &image, rectangle r, int color, bool inverted){
+
+        int width  = image.dim(0);
+        int height = image.dim(1);
+        int left, top, right, bottom;
+
         left   = (r.x0<0)  ? 0   : r.x0;
         top    = (r.y0<0)  ? 0   : r.y0;
         right  = (r.x1>=width) ? width-1 : r.x1;
         bottom = (r.y1>=height) ? height-1 : r.y1;
         if(right < left || bottom < top) return;
         int x,y;
-        for(x=left; x<=right; x++){ image(x,top)     &=color; }
-        for(x=left; x<=right; x++){ image(x,bottom)  &=color; }
-        for(y=top; y<=bottom; y++){ image(left,y)    &=color; }
-        for(y=top; y<=bottom; y++){ image(right,y)   &=color; }
-        
+        if(!inverted){
+            for(x=left; x<=right; x++){ image(x,top)     &=color; }
+            for(x=left; x<=right; x++){ image(x,bottom)  &=color; }
+            for(y=top; y<=bottom; y++){ image(left,y)    &=color; }
+            for(y=top; y<=bottom; y++){ image(right,y)   &=color; }
+        }else{
+            for(x=left; x<=right; x++){ image(x,top)     |=color; }
+            for(x=left; x<=right; x++){ image(x,bottom)  |=color; }
+            for(y=top; y<=bottom; y++){ image(left,y)    |=color; }
+            for(y=top; y<=bottom; y++){ image(right,y)   |=color; }
+        }
+    }
+
+    void paint_box_border(bytearray &image, rectangle r, byte color, bool inverted){
+
+        int width  = image.dim(0);
+        int height = image.dim(1);
+        int left, top, right, bottom;
+
+        left   = (r.x0<0)  ? 0   : r.x0;
+        top    = (r.y0<0)  ? 0   : r.y0;
+        right  = (r.x1>=width) ? width-1 : r.x1;
+        bottom = (r.y1>=height) ? height-1 : r.y1;
+        if(right < left || bottom < top) return;
+        int x,y;
+        if(!inverted){
+            for(x=left; x<=right; x++){ image(x,top)     &=color; }
+            for(x=left; x<=right; x++){ image(x,bottom)  &=color; }
+            for(y=top; y<=bottom; y++){ image(left,y)    &=color; }
+            for(y=top; y<=bottom; y++){ image(right,y)   &=color; }
+        }else{
+            for(x=left; x<=right; x++){ image(x,top)     |=color; }
+            for(x=left; x<=right; x++){ image(x,bottom)  |=color; }
+            for(y=top; y<=bottom; y++){ image(left,y)    |=color; }
+            for(y=top; y<=bottom; y++){ image(right,y)   |=color; }
+        }
     }
 
     static void subsample_boxes(narray<rectangle> &boxes, int factor) {
@@ -252,11 +313,11 @@ namespace ocropus {
 
         for(int i=0, len=boxes.length(); i<len; i++)
             paint_box_border(out, boxes[i], color);
-        
+
     }
 
     void draw_filled_rects(colib::intarray &out, colib::bytearray &in,
-                           colib::narray<colib::rectangle> &rects, 
+                           colib::narray<colib::rectangle> &rects,
                            int downsample_factor, int color, int border_color){
         int ds = downsample_factor;
         if(ds <= 0)
@@ -282,7 +343,7 @@ namespace ocropus {
             paint_box(out, boxes[i], color);
             paint_box_border(out, boxes[i], border_color);
         }
-        
+
     }
 
     // FIXME comments
@@ -318,7 +379,7 @@ namespace ocropus {
     }
 
     // FIXME comments
-    
+
     void set_version_string(const char *new_version_string) {
         if (version_string) {
             ASSERT(new_version_string && !strcmp(version_string, new_version_string));
@@ -327,6 +388,9 @@ namespace ocropus {
         }
     }
 
+    Timers::Timers() {
+    }
+
     void Timers::report() {
         fprintf(stderr,"time binarizer %g\n",*binarizer);
         fprintf(stderr,"time cleanup %g\n",*cleanup);
@@ -360,9 +424,9 @@ namespace ocropus {
     }
 
 
-    void normalize_input_classify(floatarray &feature,doublearray 
+    void normalize_input_classify(floatarray &feature,doublearray
                                   &stdev,doublearray &m_x) {
-    
+
         CHECK_ARG(stdev.length()==m_x.length());
         ASSERT(is_nan_free(m_x));
         ASSERT(is_nan_free(stdev));
@@ -519,7 +583,9 @@ namespace ocropus {
         for(int i=0;i<ids.length();i++) {
             // We MUST make gaps for spaces, otherwise the correspondence
             // between characters and segments is not well-defined anymore.
-            
+
+            // 05-06-08 IM: we don't make gaps anymore anyway
+
             // skip spaces
             if(ids[i] == 0)
                 continue;
@@ -536,11 +602,9 @@ namespace ocropus {
                 // MAY be shared between ids led to the idmap structure,
                 // otherwise we could live happily with objlist<intarray>
                 // or something.
-                #if 0
                 // segments shouldn't be shared between characters
-                if(!(cseg_to_char(cs)==-1 || cseg_to_char(cs)==chars_allocated))
-                    throw "segments are shared between multiple ids";
-                #endif
+                //if(!(cseg_to_char(cs)==-1 || cseg_to_char(cs)==chars_allocated))
+                //    throw "segments are shared between multiple ids";
                 cseg_to_char(cs) = i + 1;
             }
         }
@@ -582,61 +646,81 @@ namespace ocropus {
 
     template <class T>
     void remove_small_components(narray<T> &bimage,int mw,int mh) {
-	intarray image;
-	copy(image,bimage);
-	label_components(image);
-	narray<rectangle> rects;
-	bounding_boxes(rects,image);
-	bytearray good(rects.length());
-	for(int i=0;i<good.length();i++) 
-	    good[i] = 1;
-	for(int i=0;i<rects.length();i++) {
-	    if(rects[i].width()<mw && rects[i].height()<mh) {
-		// printf("*** %d %d %d\n",i,rects[i].width(),rects[i].height());
-		good[i] = 0;
-	    }
-	}
-	for(int i=0;i<image.length1d();i++) {
-	    if(!good(image.at1d(i)))
-		image.at1d(i) = 0;
-	}
-	for(int i=0;i<image.length1d();i++)
-	    if(!image.at1d(i)) bimage.at1d(i) = 0;
+        intarray image;
+        copy(image,bimage);
+        label_components(image);
+        narray<rectangle> rects;
+        bounding_boxes(rects,image);
+        bytearray good(rects.length());
+        for(int i=0;i<good.length();i++)
+            good[i] = 1;
+        for(int i=0;i<rects.length();i++) {
+            if(rects[i].width()<mw && rects[i].height()<mh) {
+                // printf("*** %d %d %d\n",i,rects[i].width(),rects[i].height());
+                good[i] = 0;
+            }
+        }
+        for(int i=0;i<image.length1d();i++) {
+            if(!good(image.at1d(i)))
+                image.at1d(i) = 0;
+        }
+        for(int i=0;i<image.length1d();i++)
+            if(!image.at1d(i)) bimage.at1d(i) = 0;
     }
     template void remove_small_components<byte>(narray<byte> &,int,int);
     template void remove_small_components<int>(narray<int> &,int,int);
 
     template <class T>
     void remove_marginal_components(narray<T> &bimage,int x0,int y0,int x1,int y1) {
-	x1 = bimage.dim(0)-x1;
-	y1 = bimage.dim(1)-y1;
-	intarray image;
-	copy(image,bimage);
-	label_components(image);
-	narray<rectangle> rects;
-	bounding_boxes(rects,image);
-	bytearray good(rects.length());
-	for(int i=0;i<good.length();i++) 
-	    good[i] = 1;
-	for(int i=0;i<rects.length();i++) {
-	    rectangle r = rects[i];
-#define lt <
-#define ge >
-	    if(r.x1 lt x0 || r.x0 ge x1 || r.y1 lt y0 || r.y0 ge y1) {
-		// printf("**! %d %d %d\n",i,rects[i].width(),rects[i].height());
-	        good[i] = 0;
-	    }
-	}
-	for(int i=0;i<image.length1d();i++) {
-	    if(!good(image.at1d(i)))
-		image.at1d(i) = 0;
-	}
-	for(int i=0;i<image.length1d();i++)
-	    if(!image.at1d(i)) bimage.at1d(i) = 0;
+        x1 = bimage.dim(0)-x1;
+        y1 = bimage.dim(1)-y1;
+        intarray image;
+        copy(image,bimage);
+        label_components(image);
+        narray<rectangle> rects;
+        bounding_boxes(rects,image);
+        bytearray good(rects.length());
+        fill(good, 1);
+        for(int i=0;i<rects.length();i++) {
+            rectangle r = rects[i];
+            // >= ?
+            if(r.x1 < x0 || r.x0 > x1 || r.y1 < y0 || r.y0 > y1) {
+                good[i] = 0;
+            }
+        }
+        for(int i=0;i<image.length1d();i++) {
+            if(!good(image.at1d(i)))
+                bimage.at1d(i) = 0;
+        }
     }
     template void remove_marginal_components<byte>(narray<byte> &,int,int,int,int);
     template void remove_marginal_components<int>(narray<int> &,int,int,int,int);
 
+    void remove_neighbour_line_components(bytearray &line) {
+        int h = line.dim(1);
+        invert(line);
+        intarray image;
+        copy(image,line);
+        label_components(image);
+        narray<rectangle> rects;
+        bounding_boxes(rects,image);
+        bytearray good(rects.length());
+        fill(good, 1);
+        for(int i=0;i<rects.length();i++) {
+            rectangle r = rects[i];
+            if(r.y0 == 0 && r.y1 < h / 3)
+                good[i] = 0;
+            else if(r.y1 == h && r.y0 > 2 * h / 3)
+                good[i] = 0;
+        }
+        for(int i=0;i<image.length1d();i++) {
+            if(!good(image.at1d(i)))
+                line.at1d(i) = 0;
+        }
+        invert(line);
+    }
+
+    /// Analogous to python's split().
     void split_string(narray<strbuf> &components,
                       const char *s,
                       const char *delimiters) {
@@ -651,6 +735,7 @@ namespace ocropus {
                 strbuf &item = components.push();
                 item.ensure(len + 1);
                 strncpy(item, s, len);
+                item[len] = '\0';
             }
             if(!*p) return;
             s = p + 1;
@@ -663,34 +748,61 @@ namespace ocropus {
         for(int i=0;i<image.length1d();i++)
             result.at1d(i) = image.at1d(i)<threshold ? 0 : 255;
     }
-    
-    // I think this is by Ambrish --IM
-    void runlength_histogram(narray<float> &hist, rectangle line_box, bytearray &img){
-        fill(hist,0);
-        int runlength = 0;
-        int flag = 0;
-        for(int j = line_box.y0; j < line_box.y1; j++){       
-            for(int k = line_box.x0; k < line_box.x1; k++) {
-                if (img(k,j) == 0) {
-                    runlength++;
-                    flag = 1;
-                }
-                if (img(k,j) != 0 && flag == 1) {
-                    flag = 0;
-                    if(runlength < hist.length())
-                        hist(runlength)++;
-                    runlength = 0;
-                }
-            }
-        }    
-    }   
 
-    void runlength_histogram(narray<float> &hist, bytearray &img) {
-        CHECK_ARG(background_seems_white(img));
-        runlength_histogram(hist, rectangle(0, 0, img.dim(0), img.dim(1)), img);
+    void binarize_simple(colib::bytearray &image) {
+        binarize_simple(image, image);
     }
-    
-    int find_median(narray<float> &hist){
+
+    void runlength_histogram(floatarray &hist,
+                             bytearray &img,
+                             rectangle box,
+                             bool white,
+                             bool vertical){
+        fill(hist,0);
+        white = !!white;
+        if(!vertical) {
+            for(int j=box.y0;j<box.y1;j++){
+                int flag = 0;
+                int runlength = 0;
+                for(int k=box.x0;k<box.x1;k++) {
+                    if (!!img(k,j)==white) {
+                        runlength++;
+                        flag = 1;
+                    }
+                    if (!!img(k,j) != white && flag) {
+                        flag = 0;
+                        if(runlength < hist.length())
+                            hist(runlength)++;
+                        runlength = 0;
+                    }
+                }
+                if (flag && runlength < hist.length())
+                    hist(runlength)++;
+            }
+        } else {
+            for(int k=box.x0;k<box.x1;k++) {
+                int flag = 0;
+                int runlength = 0;
+                for(int j=box.y0;j<box.y1;j++){
+                    if (!!img(k,j)==white) {
+                        runlength++;
+                        flag = 1;
+                    }
+                    if (!!img(k,j) != white && flag) {
+                        flag = 0;
+                        if(runlength < hist.length())
+                            hist(runlength)++;
+                        runlength = 0;
+                    }
+                }
+                if (flag && runlength < hist.length())
+                    hist(runlength)++;
+            }
+        }
+    }
+
+    /// Return the index of the histogram median.
+    int find_median_in_histogram(narray<float> &hist){
         int index= 0;
         float partial_sum = 0, sum = 0 ;
         for(int i = 0; i < hist.length(); i++) sum += hist(i);
@@ -705,9 +817,53 @@ namespace ocropus {
     void throw_fmt(const char *format, ...) {
         va_list v;
         va_start(v, format);
-        static char buf[1000]; 
-        vsprintf(buf, format, v); // XXX: that's unsafe
+        static char buf[1000];
+        vsnprintf(buf, sizeof(buf), format, v);
         va_end(v);
         throw (const char *) buf;
     }
+
+    void optional_check_background_is_darker(colib::bytearray &a) {
+        if(bgcheck) {
+            CHECK_CONDITION(background_seems_black(a));
+        }
+    }
+    void optional_check_background_is_lighter(colib::bytearray &a) {
+        if(bgcheck) {
+            CHECK_CONDITION(background_seems_white(a));
+        }
+    }
+
+    void paint_rectangles(intarray &image,rectarray &rectangles) {
+        int w = image.dim(0), h = image.dim(1);
+        for(int i=0;i<rectangles.length();i++) {
+            rectangle r = rectangles[i];
+            r.x0 = r.x0<0?0:r.x0;
+            r.y0 = r.y0<0?0:r.y0;
+            r.x1 = r.x1>=w?w-1:r.x1;
+            r.y1 = r.y1>=h?h-1:r.y1;
+            int color = i+1;
+            for(int x=r.x0;x<r.x1;x++)
+                for(int y=r.y0;y<r.y1;y++)
+                    image(x,y) = color;
+        }
+    }
+
+    void strbuf_format(strbuf &str, const char *format, ...) {
+        strbuf result;
+        int len = 1;
+        result.ensure(len);
+        va_list v;
+        va_start(v, format);
+        int written = vsnprintf(result, len + 1, format, v);
+        va_end(v);
+        if(written > len) {
+            len = written;
+            result.ensure(len);
+            va_start(v, format);
+            written = vsnprintf(result, len + 1, format, v);
+            va_end(v);
+        }
+        str = result;
+    }
 }
