@@ -32,6 +32,10 @@
 #include "colib.h"
 
 namespace ocropus {
+    void count_neighbors(colib::bytearray &result,colib::bytearray &image);
+    void find_endpoints(colib::bytearray &result,colib::bytearray &image);
+    void find_junctions(colib::bytearray &result,colib::bytearray &image);
+    void remove_singular_points(colib::bytearray &image,int d);
     colib::ISegmentLine *make_CurvedCutSegmenter();
     colib::ISegmentLine *make_ConnectedComponentSegmenter();
     colib::ISegmentLine *make_SkelSegmenter();
