@@ -18,7 +18,7 @@ tmpImage = bytearray:new()
 pageseg = intarray:new()
 line_image = bytearray:new()
 line_text = nustring:new()
-line_boxes = rectanglearray:new()
+line_boxes = rectarray:new()
 line_costs = floatarray:new()
 
 pages = Pages:new()
