@@ -28,9 +28,17 @@ extern "C" {
 #include <signal.h>
 }
 
+#ifdef GOOGLE_INTERNAL
+#include "file/base/file.h"
+#include "base/google.h"
+#endif
+
 #include "ocrotoplevel.h"
 
 int main(int argc,char **argv) {
+#ifdef GOOGLE_INTERNAL
+    File::Init();
+#endif
     ocroscript_init(); 
     // ocroscript_init() returns lua_State * if you need it
     // put your own library initializations here
