@@ -1,4 +1,5 @@
 require 'lib.util'
+require 'lib.dataset'
 
 if #arg ~= 3 then
     print(arg[0].." - evaluate a bpnet classifier on a list of words")
@@ -28,7 +29,7 @@ errors = 0
 curent_nb = 0
 max_number = 1e30
 log = io.open(arg[3],"w")
-for line, gt in dataset_entries(arg[1]) do
+for line, gt in datasets.dataset_entries(arg[1]) do
     gt = plain_text_from_transcript(gt)
     fst = make_StandardFst()
     ocr:recognizeLine(fst, line)
