@@ -2,7 +2,13 @@
 --
 -- Example of using the Tesseract line recognizer together with 
 -- OCRopus layout analysis.
--- Note: for better recognition, see rec-tess.lua
+-- Note: this does not work very well right now because Tesseract
+-- has problems with recognizing text in individual lines.
+
+import_all(ocr)
+import_all(tesseract)
+import_all(nustring)
+
 
 if #arg < 1 then
     arg = { "../../data/pages/alice_1.png" }
@@ -15,7 +21,7 @@ segmenter = make_SegmentPageByRAST()
 page_image = bytearray()
 page_segmentation = intarray()
 line_image = bytearray()
-bboxes = rectanglearray()
+bboxes = rectarray()
 costs = floatarray()
 tesseract_recognizer = make_TesseractRecognizeLine()
 
