@@ -7,7 +7,7 @@ require 'lib.util'
 
 if #arg != 1 then
     print("Usage: ocroscript "..arg[0].." <binary-image.png>")
-    exit(1)
+    os.exit(1)
 end
 
 image = read_image_gray_checked(arg[1])
