@@ -1,5 +1,6 @@
-
 --dinit(1000,1000)
+import_all(ocr)
+import_all(graphics)
 
 dofile("utest.lua")
 
@@ -24,15 +25,15 @@ function try_deskewing(file)
     print(string.format("Testing image: %s",file))
     image = bytearray:new()
     tolua.takeownership(image)
-    read_image_gray(image,file)
+    iulib.read_image_gray(image,file)
     dshow(image,"a")
 
-    system(string.format("convert -rotate %d %s rotated.png",skewangles[count],file))
+    os.execute(string.format("convert -rotate %d %s rotated.png",skewangles[count],file))
     count = count + 1
 
     rotated = bytearray:new()
     tolua.takeownership(rotated)
-    read_image_gray(rotated,"rotated.png")
+    iulib.read_image_gray(rotated,"rotated.png")
     dshow(rotated,"b")
 
     result = bytearray:new()
