@@ -1,3 +1,5 @@
+import_all(ocr)
+import_all(graphics)
 
 --  dinit(1000,1000)
 
@@ -24,20 +26,17 @@ images = {
 }
 
 function try_txtimgseg(file)
-    image = bytearray:new()
-    tolua.takeownership(image)
-    read_image_binary(image,file)
+    image = bytearray()
+    iulib.read_image_binary(image,file)
          
     note("Original result")
     result = outimage(file)
-    resultimage = bytearray:new()
-    tolua.takeownership(resultimage)
-    read_image_binary(resultimage,result)
+    resultimage = bytearray()
+    iulib.read_image_binary(resultimage,result)
     dshow(resultimage,"a")
 
     note("Obtained result")
-    seg = bytearray:new()
-    tolua.takeownership(seg)
+    seg = bytearray()
     narray.fill(seg,0)
     noiseremoval:cleanup(seg,image)
     dshow(seg,"b")
