@@ -1,9 +1,11 @@
+import_all(ocr)
+import_all(graphics)
 
 -- dinit(1000,1000)
 
 dofile("utest.lua")
 
-txtimgseg = make_TextImageSeg()
+txtimgseg = make_TextImageSegByLeptonica()
 
 function outimage(file)
     -- note: order matches for these patterns
@@ -19,27 +21,30 @@ images = {
 "cleanup-images/S03LBIN-clean.png",
 }
 
+logger = Logger("foo")
 function try_txtimgseg(file)
     print(string.format("Testing image: %s",file))
     image = bytearray:new()
     tolua.takeownership(image)
-    read_image_binary(image,file)
+    iulib.read_image_binary(image,file)
          
     note("Original result")
     result = outimage(file)
-    resultimage = bytearray:new()
-    tolua.takeownership(resultimage)
-    read_image_binary(resultimage,result)
+    resultimage = bytearray()
+    iulib.read_image_binary(resultimage,result)
     dshow(resultimage,"a")
 
     note("Obtained result")
-    seg = bytearray:new()
-    tolua.takeownership(seg)
-    narray.fill(seg,0)
-    txtimgseg:cleanup(seg,image)
-    dshow(seg,"b")
+    probabilities = intarray()
+    narray.fill(probabilities, 0)
+    txtimgseg:textImageProbabilities(probabilities,image)
+    dshow(probabilities,"b")
     dwait()
 
+    seg = bytearray()
+    narray.copy(seg, probabilities)
+    logger:log("seg", seg)
+    logger:log("resultimage", resultimage)
     test_assert(narray.equal(seg,resultimage),"result differs from original")
 end
 
