@@ -1,3 +1,4 @@
+import_all(graphics)
 -- -*- lua -*-
 
 dofile("utest.lua")
@@ -6,15 +7,15 @@ dofile("utest.lua")
 
 line_image = bytearray:new() line_seg = intarray:new()
 
-read_image_gray(line_image,"images/line.png")
-make_page_black(line_image)
+iulib.read_image_gray(line_image,"images/line.png")
+iulib.make_page_black(line_image)
 
 dshow(line_image,"y")
 
-linesegmenter = make_ConnectedComponentSegmenter()
+linesegmenter = ocr.make_ConnectedComponentSegmenter()
 linesegmenter:charseg(line_seg,line_image)
-grouper = make_StandardGrouper()
-sort_by_xcenter(line_seg)
+grouper = ocr.make_StandardGrouper()
+ocr.sort_by_xcenter(line_seg)
 grouper:setSegmentation(line_seg)
 
 char = bytearray:new()
