@@ -28,6 +28,6 @@ test_failure(function() b:at(0) end,"catching 1D access of 2D array")
 -- test_assert(samedims(a,b))
 -- test_assert(equal(a,b))
 section "read_image_gray"
-read_image_gray(b,"images/simple.png")
+iulib.read_image_gray(b,"images/simple.png")
 test_assert(b:dim(0)==640 and b:dim(1)==480)
 collectgarbage()
